package xgs.net.init;

import org.slf4j.*;
import com.jfinal.json.*;
import xgs.net.logs.*;
import com.jfinal.log.*;
import com.jfinal.template.*;
import com.jfinal.plugin.*;
import com.jfinal.plugin.activerecord.*;
import xgs.net.model.*;
import com.jfinal.plugin.redis.*;
import com.jfinal.config.*;
import com.jfinal.handler.*;
import com.jfinal.plugin.druid.*;
import xgs.net.session.*;
import xgs.net.handle.*;
import com.jfinal.kit.*;

public abstract class XgsConfig extends JFinalConfig
{
    private Logger logger = LoggerFactory.getLogger(this.getClass());
    protected Routes routes;
    public static Prop prop;


    public void configConstant(final Constants me) {
        me.setDevMode(XgsConfig.prop.getBoolean("system.dev", false));
        me.setJsonFactory(new MixedJsonFactory());
        me.setLogFactory(new SLF4JLogFactory());
        PathKit.setWebRootPath("/aa");
        this.constant(me);
    }

    public abstract void constant(final Constants p0);

    public void configRoute(final Routes me) {
        this.routes = me;
        this.logger.debug("init routes");
    }

    public void configEngine(final Engine me) {
        System.err.println("\u8fdb\u5165configEngine:>>>" + me.getBaseTemplatePath());
    }

    public void configPlugin(final Plugins me) {
        this.logger.debug("init plugins");
        try {
            this.logger.debug("init database..");
            final InitDb db = (InitDb)Class.forName(XgsConfig.prop.get("system.db")).newInstance();
            final DruidPlugin druidPlugin = db.init();
            druidPlugin.setMaxPoolPreparedStatementPerConnectionSize(InitDb.prop.getInt("jdbc.min_poll", 10));
            druidPlugin.setMaxActive(InitDb.prop.getInt("jdbc.max_poll", 100));
            me.add(druidPlugin);
            final ActiveRecordPlugin arp = new ActiveRecordPlugin(druidPlugin);
            arp.setShowSql(InitDb.prop.getBoolean("jdbc.show_sql", false));
            arp.setContainerFactory(new CaseInsensitiveContainerFactory(true));
            arp.setDialect(db.getDialect());
            _MappingKit.mapping(arp);
            me.add(arp);
            if (XgsConfig.prop.getBoolean("cache.enable", false)) {
                me.add(new RedisPlugin(XgsConfig.prop.get("cache.redis.wx.name"), XgsConfig.prop.get("cache.redis.host"), XgsConfig.prop.getInt("cache.redis.port")));
            }
            this.logger.debug("inited database..");
        }
        catch (Exception e) {
            this.logger.error(e.getMessage());
            e.printStackTrace();
        }
        this.plugin(me);
    }

    public abstract void plugin(final Plugins p0);

    public void configInterceptor(final Interceptors me) {
        this.interceptor(me);
    }

    public abstract void interceptor(final Interceptors p0);

    public void configHandler(final Handlers me) {
        me.add(new RenderingTimeHandler());
        me.add(new DruidStatViewHandler("/druid"));
        if (XgsConfig.prop.getBoolean("session.enable", false)) {
            SessionRepositoryRequestWrapper.initCfg(XgsConfig.prop.get("session.domain"), XgsConfig.prop.getInt("session.maxAgeInSeconds"), XgsConfig.prop.get("session.id"), XgsConfig.prop.get("session.cache"));
            me.add(new SessionHandler());
        }
        this.handler(me);
    }

    public abstract void handler(final Handlers p0);

    static {
        XgsConfig.prop = PropKit.use("cfg.properties");
    }
}
