package xgs.net.interceptors;

import com.jfinal.aop.Interceptor;
import com.jfinal.aop.Invocation;
import com.jfinal.core.Controller;
import com.jfinal.kit.JsonKit;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import xgs.net.annotation.Log;

import java.lang.reflect.Parameter;
import java.util.HashMap;
import java.util.Map;

public class LogInterceptor implements Interceptor
{
    Logger logger = LoggerFactory.getLogger((Class)this.getClass());

    
    public void intercept(final Invocation inv) {
        try {
            inv.invoke();
            this.print(inv);
        }
        catch (Exception e) {
            this.print(inv, e);
            throw e;
        }
    }
    
    private void print(final Invocation inv, final Throwable ex) {
        new Thread(() -> {
            HashMap<String, String> map;
            Object args2;
            Parameter[] parameters;
            String keyWord;
            int i;
            Log log = inv.getMethod().getAnnotation(Log.class);
            if (log != null) {
                Object args = null;
                if (inv.getTarget() instanceof Controller) {
                    args2 = inv.getController().getRequest().getParameterMap();
                }
                else {
                    parameters = inv.getMethod().getParameters();
                    args2 = new HashMap();
                    for (i = 0; i < parameters.length; ++i) {
                        ((Map)args2).put(parameters[i].getName(), inv.getArg(i));
                    }
                }
                keyWord = log.value();
                if (log.value().isEmpty()) {
                    keyWord = inv.getTarget().getClass().getSimpleName() + " > " + inv.getMethodName();
                }
                map = new HashMap<>();
                map.put("className", inv.getTarget().getClass().getName());
                map.put("methodName", inv.getMethodName());
                map.put("keyword", keyWord);
                map.put("args", (String)args2);
                if (ex != null) {
                    map.put("exception", ExceptionUtils.getFullStackTrace(ex));
                }
                this.logger.info(JsonKit.toJson(map));
            }
        }).start();
    }
    
    private void print(Invocation inv) {
        this.print(inv, null);
    }
}
