package xgs.net.query.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import xgs.net.query.Filter;
import xgs.net.query.FilterBuilder;
import xgs.net.utils.StrUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class LigerFilter implements Filter
{
    private static Logger log;
    private static String blank;
    private static Map<String, String> opration;
    private static Map<String, String> params;
    private static LigerFilter instance;
    
    public static LigerFilter getInstance() {
        if (LigerFilter.instance == null) {
            LigerFilter.instance = new LigerFilter();
        }
        return LigerFilter.instance;
    }
    
    private Entity factory(final JSONObject obj) {
        final StringBuilder sb = new StringBuilder();
        final String op = obj.getString(fields.op.name());
        final String value = obj.getString(fields.value.name());
        String param = obj.getString(value);
        sb.append(obj.getString(fields.field.name()));
        sb.append(LigerFilter.blank);
        if (LigerFilter.opration.containsKey(op)) {
            sb.append(LigerFilter.blank);
            sb.append(LigerFilter.opration.get(op));
            param = LigerFilter.params.get(op).replace("?", value);
            final Entity entity = new Entity();
            entity.setParam(param);
            entity.setSnippets(sb.toString());
            return entity;
        }
        return null;
    }
    
    private Entity builder(JSONObject obj) {
        final StringBuilder sb = new StringBuilder();
        sb.append(LigerFilter.blank);
        sb.append(rule.and.name());
        sb.append(LigerFilter.blank);
        final Entity entity = this.factory(obj);
        sb.append(entity.getSnippets());
        entity.setSnippets(sb.toString());
        return entity;
    }
    
    public HashMap<String, Object> parsetoMap(String text) {
        if (StrUtils.isBlank(text)) {
            return null;
        }
        final JSONObject object = JSON.parseObject(text);
        final JSONArray list = (JSONArray)object.get("rules");
        final HashMap<String, Object> map = new HashMap<>();
        for (int i = 0, len = list.size(); i < len; ++i) {
            final JSONObject obj = list.getJSONObject(i);
            map.put(obj.getString(fields.field.name()), obj.getString(fields.value.name()));
        }
        return map;
    }
    
    @Override
    public FilterBuilder parse(String text) {
        FilterBuilder fe = new FilterBuilder();
        if (StrUtils.notBlank(text)) {
            JSONObject object = JSON.parseObject(text);
            JSONArray list = (JSONArray)object.get("rules");
            StringBuilder sb = new StringBuilder();
            List<Object> params = new ArrayList<>();
            for (int i = 0, len = list.size(); i < len; ++i) {
                JSONObject obj = list.getJSONObject(i);
                Entity entity = this.builder(obj);
                sb.append(entity.getSnippets());
                params.add(entity.getParam());
            }
            fe.setParams(params);
            fe.setSnippets(sb.toString());
        }
        LigerFilter.log.debug(JSON.toJSONString(fe));
        return fe;
    }
    
    public static void main(final String[] args) {
        final String where = "{\"rules\":[{\"field\":\"user_name\",\"op\":\"equal\",\"value\":\"aa\",\"type\":\"text\"},{\"field\":\"state\",\"op\":\"equal\",\"value\":\"1\",\"type\":\"select\"}]}  ";
        final LigerFilter filter = getInstance();
        filter.parse(where);
    }
    
    static {
        LigerFilter.log = LoggerFactory.getLogger(LigerFilter.class);
        LigerFilter.blank = " ";
        (LigerFilter.opration = new HashMap<>()).put("equal", " = ? ");
        LigerFilter.opration.put("notequal", " <> ? ");
        LigerFilter.opration.put("startwith", " like ? ");
        LigerFilter.opration.put("endwith", " like ? ");
        LigerFilter.opration.put("like", " like ? ");
        LigerFilter.opration.put("greater", " > ? ");
        LigerFilter.opration.put("greaterorequal", " >= ? ");
        LigerFilter.opration.put("less", " < ? ");
        LigerFilter.opration.put("lessorequal", " <= ? ");
        LigerFilter.opration.put("in", "like ? ");
        LigerFilter.opration.put("notin", " not like ? ");
        (LigerFilter.params = new HashMap<>()).put("equal", "?");
        LigerFilter.params.put("notequal", "?");
        LigerFilter.params.put("startwith", "?%");
        LigerFilter.params.put("endwith", "%?");
        LigerFilter.params.put("like", "%?%");
        LigerFilter.params.put("greater", "?");
        LigerFilter.params.put("greaterorequal", "?");
        LigerFilter.params.put("less", "?");
        LigerFilter.params.put("lessorequal", "?");
        LigerFilter.params.put("in", "%?%");
        LigerFilter.params.put("notin", "%?%");
    }
    
    private enum fields
    {
        field, 
        op, 
        value, 
        type;
    }
    
    private enum rule
    {
        and, 
        or;
    }
    
    class Entity
    {
        private String param;
        private String snippets;
        
        public String getParam() {
            return this.param;
        }
        
        public void setParam(final String param) {
            this.param = param;
        }
        
        public String getSnippets() {
            return this.snippets;
        }
        
        public void setSnippets(final String snippets) {
            this.snippets = snippets;
        }
    }
}
