package xgs.net.service;

import com.jfinal.aop.Before;
import com.jfinal.plugin.activerecord.Db;
import com.jfinal.plugin.activerecord.Model;
import com.jfinal.plugin.activerecord.Table;
import com.jfinal.plugin.activerecord.TableMapping;
import com.jfinal.plugin.activerecord.tx.Tx;
import com.jfinal.plugin.redis.Cache;
import com.jfinal.plugin.redis.Redis;
import xgs.net.annotation.Entity;
import xgs.net.annotation.Inject;
import xgs.net.controller.BaseController;
//import xgs.net.model.TbUser;
import xgs.net.token.JwtToken;

import java.util.List;

public class BaseService<T extends Model>
{

    private static final String userCoustmerSession = "group";
    private static final int expire = 7*60*60*24;
    private static final Cache redis = Redis.use();
    public static final int batchSize = 1000;
    @Inject
    public T dao;
    private static final String deleteByIds = "delete from %s where id = ? ";
    private static final String findAllSql = "select * from %s";
    private static final String AddPraiseSql = "update %s set praise = praise+1  where id = ?";
    private static final String MinusPraiseSql = "update %s set praise = praise-1  where id = ?";
    private static final String AddFavoriteSql = "  update %s set favorite = favorite+1 where id = ?";
    private static final String MinusFavoriteSql = "  update %s set favorite = favorite-1 where id = ?";
    private static final String AddCommentSql = "  update %s set comment = comment+1 where id = ?";
    private static final String MinusCommentSql = "  update %s set comment = comment-1 where id = ?";
    private static final String isFavoriteSql = "select count(1) from tb_favorite where trande_id = ? and user_id = ?";
    private static final String isPraiseSql = "select count(1) from tb_praise where trande_id = ? and user_id = ?";

    protected Entity getEntity() {
        return this.dao.getClass().getAnnotation(Entity.class);
    }
    
    protected String tableName() {
        return this.dao.getClass().getAnnotation(Entity.class).value();
    }
    
    protected Table getTable() {
        return TableMapping.me().getTable(this.dao.getClass());
    }
    
    public T findById(Object id) {
        return (T)dao.findById(id);
    }
    
    public void deleteById(final Object id) {
        this.dao.deleteById(id);
    }
    
    protected Object[][] assembleIds(final Object... ids) {
        final Object[][] temps = new Object[ids.length][1];
        final int i = 0;
        for (final Object id : ids) {
            temps[i][0] = id;
        }
        return temps;
    }

//    protected void cacheToken(String JwtToken, TbUser user) {
//        BaseService.redis.setex(JwtToken, expire, user);
//    }

    protected Object[][] assembleIds(final Object id, final Object... ids) {
        final Object[][] temps = new Object[ids.length][2];
        final int i = 0;
        for (final Object _id : ids) {
            temps[i][0] = _id;
            temps[i][1] = id;
        }
        return temps;
    }
    
    public void deleteByIds(Object... ids) {
        for (Object id : ids) {
            Db.update(String.format(deleteByIds, this.tableName()), id);
        }
    }
    
    public List<T> findAll() {
        return this.dao.find(String.format(findAllSql, this.tableName()));
    }
    
    @Before({ Tx.class })
    public void AddPraise(Long id) {
        Db.update(String.format(AddPraiseSql, this.tableName()), id);
    }
    
    @Before({ Tx.class })
    public void MinusPraise(Long id) {
        Db.update(String.format(MinusPraiseSql, this.tableName()), id);
    }
    
    @Before({ Tx.class })
    public void AddFavorite(Long id) {
        Db.update(String.format(AddFavoriteSql, this.tableName()), id);
    }
    
    @Before({ Tx.class })
    public void MinusFavorite(Long id) {
        Db.update(String.format(MinusFavoriteSql, this.tableName()), id);
    }
    
    @Before({ Tx.class })
    public void AddComment(Long id) {
        Db.update(String.format(AddCommentSql, this.tableName()), id);
    }
    
    @Before({ Tx.class })
    public void MinusComment(Long id) {
        Db.update(String.format(MinusCommentSql, this.tableName()), id);
    }

    
    public int isFavorite(Long trandeId, Long userId) {
        return Db.queryInt(isFavoriteSql, trandeId, userId);
    }
    
    public int isPraise(Long trandeId, Long userId) {
        return Db.queryInt(isPraiseSql, trandeId, userId);
    }


    protected String getToken(Long id){
        String token = JwtToken.createToken(id);
        return  userCoustmerSession + "-" + token;
    }

    protected void clearToken(String token){
        redis.expire(token,0);
    }
}
