package xgs.net.session;

import com.jfinal.core.JFinal;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpSessionContext;
import java.io.Serializable;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

class Session implements HttpSession, Serializable
{
    private static final long serialVersionUID = -719739616470614546L;
    protected String id;
    protected final Map<String, Object> attributes;
    protected final long creationTime;
    protected volatile int maxInactiveInterval;
    protected volatile boolean isNew;
    protected transient SessionManager manager;
    
    public Session(final String id) {
        this.id = null;
        this.attributes = new ConcurrentHashMap<String, Object>();
        this.creationTime = System.currentTimeMillis();
        this.maxInactiveInterval = -1;
        this.isNew = false;
        this.id = id;
    }
    
    public Object getAttribute(final String key) {
        return this.attributes.get(key);
    }
    
    public Enumeration getAttributeNames() {
        final Set<String> names = new HashSet<String>();
        names.addAll(this.attributes.keySet());
        return Collections.enumeration(names);
    }
    
    public long getCreationTime() {
        return this.creationTime;
    }
    
    public String getId() {
        return this.id;
    }
    
    public long getLastAccessedTime() {
        return this.creationTime;
    }
    
    public int getMaxInactiveInterval() {
        return this.getMaxInactiveInterval();
    }
    
    public ServletContext getServletContext() {
        return JFinal.me().getServletContext();
    }
    
    @Deprecated
    public HttpSessionContext getSessionContext() {
        return null;
    }
    
    @Deprecated
    public Object getValue(final String key) {
        return this.getAttribute(key);
    }
    
    @Deprecated
    public String[] getValueNames() {
        return this.attributes.keySet().toArray(new String[0]);
    }
    
    public void invalidate() {
    }
    
    public boolean isNew() {
        return this.isNew;
    }
    
    @Deprecated
    public void putValue(final String key, final Object value) {
        this.setAttribute(key, value);
    }
    
    public synchronized void removeAttribute(final String key) {
        this.attributes.remove(key);
        this.manager.update(this);
    }
    
    @Deprecated
    public void removeValue(final String key) {
        this.removeAttribute(key);
    }
    
    public synchronized void setAttribute(final String key, final Object value) {
        if (key == null) {
            throw new IllegalArgumentException("setAttribute: name parameter cannot be null");
        }
        if (value == null) {
            this.removeAttribute(key);
            return;
        }
        this.attributes.put(key, value);
        this.manager.update(this);
    }
    
    public void setMaxInactiveInterval(final int interval) {
        this.maxInactiveInterval = interval;
    }
    
    public synchronized void setManager(final SessionManager manager) {
        this.manager = manager;
    }
}
