package xgs.net.session;

import com.jfinal.kit.StrKit;
import xgs.net.utils.StrUtils;
import xgs.net.utils.WebUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

public class SessionRepositoryRequestWrapper extends HttpServletRequestWrapper
{
    private static SessionManager sessionManager;
    private static String sessionCookieDomain;
    private static String sessionCookieName;
    private static int maxAgeInSeconds;
    private final HttpServletResponse response;

    public static void initCfg(final String sessionCookieDomain, final int maxAgeInSeconds, final String sessionId, final String cacheName) {
        SessionRepositoryRequestWrapper.sessionCookieDomain = sessionCookieDomain;
        SessionRepositoryRequestWrapper.maxAgeInSeconds = maxAgeInSeconds * 60;
        SessionRepositoryRequestWrapper.sessionCookieName = sessionId;
        SessionRepositoryRequestWrapper.sessionManager = new RedisSessionManager(cacheName);
    }

    public SessionRepositoryRequestWrapper(final HttpServletRequest request, final HttpServletResponse response) {
        super(request);
        this.response = response;
    }

    private HttpServletRequest getHttpRequest() {
        return (HttpServletRequest)super.getRequest();
    }

    private String getSessionId() {
        final HttpServletRequest request = this.getHttpRequest();
        return WebUtils.getCookie(request, SessionRepositoryRequestWrapper.sessionCookieName);
    }

    public String getRequestedSessionId() {
        return this.getSessionId();
    }

    public HttpSession getSession() {
        String sessionId = this.getRequestedSessionId();
        if (null == sessionId) {
            sessionId = StrUtils.getRandomUUID();
            WebUtils.setCookie(this.response, SessionRepositoryRequestWrapper.sessionCookieName, sessionId, SessionRepositoryRequestWrapper.sessionCookieDomain, SessionRepositoryRequestWrapper.maxAgeInSeconds);
        }
        Session session = SessionRepositoryRequestWrapper.sessionManager.get(sessionId);
        if (null == session) {
            session = new Session(sessionId);
            session.setMaxInactiveInterval(SessionRepositoryRequestWrapper.maxAgeInSeconds);
            SessionRepositoryRequestWrapper.sessionManager.save(session);
        }
        else {
            session.setMaxInactiveInterval(SessionRepositoryRequestWrapper.maxAgeInSeconds);
            SessionRepositoryRequestWrapper.sessionManager.update(session);
        }
        session.setManager(SessionRepositoryRequestWrapper.sessionManager);
        return (HttpSession)session;
    }

    public HttpSession getSession(final boolean create) {
        if (create) {
            return this.getSession();
        }
        final String sessionId = this.getRequestedSessionId();
        if (null == sessionId) {
            return null;
        }
        final Session session = SessionRepositoryRequestWrapper.sessionManager.get(sessionId);
        session.setMaxInactiveInterval(SessionRepositoryRequestWrapper.maxAgeInSeconds);
        SessionRepositoryRequestWrapper.sessionManager.update(session);
        if (null != session) {
            session.setManager(SessionRepositoryRequestWrapper.sessionManager);
        }
        return (HttpSession)session;
    }

    public boolean isRequestedSessionIdValid() {
        final String sessionId = this.getRequestedSessionId();
        return StrKit.notBlank(sessionId);
    }

    static {
        SessionRepositoryRequestWrapper.sessionCookieName = "PHPSESSION";
    }
}
