package xgs.net.controller;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import xgs.net.annotation.ControllerKey;
import xgs.net.common.BaseController;
import xgs.net.common.Constants;
import xgs.net.exception.XgsException;
import xgs.net.model.FileUpload;
import xgs.net.plugin.ioc.IocKit;
import xgs.net.service.FileServices;
import xgs.net.util.ImageUtil;
import xgs.net.utils.StrUtils;

import java.io.File;

@ControllerKey("/image")
public class ImageController
    extends BaseController {
    Logger logger = LoggerFactory.getLogger(ImageController.class);
    FileServices fileServices = IocKit.getBean(FileServices.class);

    private File getImage() {
        Long id = getParaToLong(0);
        FileUpload fu = this.fileServices.findById(id);
        if ((fu == null) || (StrUtils.isBlank(fu.getFilePath()))) {
            throw new XgsException(Constants.error.getInt("file.notfileinfo"));
        }
        String filePath = fu.getFilePath();
        File src = new File(filePath);
        getResponse().setHeader("Content-Type", fu.getFileType());
        return src;
    }

    private void renderImage(File file){
        this.renderFile(file);
//        FileInputStream hFile = new FileInputStream(file);
//
//        int i = hFile.available();
//        byte[] data = new byte[i];
//
//        hFile.read(data);
//
//        ServletOutputStream sos = getResponse().getOutputStream();
//
//        sos.write(data);
//        sos.flush();
//        sos.close();
//        hFile.close();
//        renderNull();
    }

    public void scale() {
        try {
            Integer height = getParaToInt(1);
            Integer width = getParaToInt(2);
            File src = getImage();
            File file = ImageUtil.scaleSize(src, height, width);
            renderImage(file);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void index() {
        try {
            Long start = System.currentTimeMillis();
            Integer height = getParaToInt(1);
            Integer width = getParaToInt(2);
            File src = getImage();
            if (!src.exists()) {
                throw new XgsException(Constants.error.getInt("file.notExist"));
            }
            File file;
            if ((height != null) && (width != null)) {
                file = ImageUtil.regionCenter(src, height, width);
            } else {
                if ((height == null) && (width == null)) {
                    file = src;
                } else {
                    file = ImageUtil.scale(src, height, width);
                }
            }
            Long end1 = System.currentTimeMillis();
            renderImage(file);
            Long end2 = System.currentTimeMillis();
            logger.info("====================================");
            logger.info(this.getPara(0));
            logger.info("处理图片时间"+(end1-start));
            logger.info("输出图片时间"+(end2-end1));
            logger.info("====================================");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
