package xgs.net.controller;

import com.jfinal.aop.Before;
import com.jfinal.upload.UploadFile;
import xgs.net.annotation.ControllerKey;
import xgs.net.common.BaseController;
import xgs.net.common.Constants;
import xgs.net.exception.XgsException;
import xgs.net.interceptor.FileInterceptor;
import xgs.net.model.FileUpload;
import xgs.net.plugin.ioc.IocKit;
import xgs.net.service.FileServices;
import xgs.net.utils.StrUtils;

import java.io.File;
import java.io.IOException;
import java.util.List;

@ControllerKey("/")
public class IndexController
    extends BaseController {
    FileServices fileServices = (FileServices) IocKit.getBean(FileServices.class);

    @Before({FileInterceptor.class})
    public void uploads() {
        List<UploadFile> uploadFiles = null;
        try {
            uploadFiles = getFiles();
        } catch (Exception e) {
            throw new XgsException(Constants.error.getInt("file.notupload"));
        }
        List<Long> ids = this.fileServices.saveFiles(uploadFiles);
        this.msg.setMsg("上传成功");
        this.msg.setData(ids);
        renderJson(this.msg);
    }

    @Before({FileInterceptor.class})
    public void wxupload() throws IOException {
        Long id = getParaToLong("id");
        UploadFile uploadFile = getFile();
        try {
            this.fileServices.save(uploadFile, id);
            this.msg.setData(id);
            this.msg.setMsg("上传成功");
            renderJson(this.msg);
        } catch (Exception e) {
            throw new XgsException(Constants.error.getInt("file.notupload"));
        }
    }

    public void base64() {
        try {
            String base64 = getPara("base64");
            Long id = this.fileServices.save(base64);
            this.msg.setData(id);
            this.msg.setMsg("上传成功");
            renderJson(this.msg);
        } catch (Exception e) {
            throw new XgsException(Constants.error.getInt("file.notupload"));
        }
    }

    @Before({FileInterceptor.class})
    public void index() {
        UploadFile uploadFile = null;
        try {
            uploadFile = getFile();
        } catch (Exception e) {
            throw new XgsException(Constants.error.getInt("file.notupload"));
        }
        Long id = this.fileServices.save(uploadFile);
        this.msg.setMsg("上传成功");
        this.msg.setData(id);
        renderJson(this.msg);
    }

    public void detail() {
        Long id = getParaToLong();
        FileUpload fu = this.fileServices.findById(id);
        if (fu == null) {
            throw new XgsException(Constants.error.getInt("file.notfileinfo"));
        }
        fu.setFilePath(null);
        fu.removeNullValueAttrs();
        this.msg.setData(fu);
        renderJson(this.msg);
    }

    public void download() {
        Long id = getParaToLong();
        FileUpload fu = this.fileServices.findById(id);
        if ((fu == null) || (StrUtils.isBlank(fu.getFilePath()))) {
            throw new XgsException(Constants.error.getInt("file.notfileinfo"));
        }
        String filePath = fu.getFilePath();
        File file = new File(filePath);
        renderFile(file);
    }
}
