package xgs.net.service;

import com.jfinal.aop.Before;
import com.jfinal.plugin.activerecord.tx.Tx;
import com.jfinal.upload.UploadFile;
import xgs.net.Base64Entity;
import xgs.net.annotation.Service;
import xgs.net.common.Constants;
import xgs.net.exception.XgsException;
import xgs.net.model.FileUpload;
import xgs.net.util.Base64Util;
import xgs.net.util.FileUtil;
import xgs.net.utils.IdGen;
import xgs.net.utils.MD5Utils;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.List;

@Service
public class FileServices
  extends BaseServices
{
  public FileUpload findById(Long id)
  {
    return FileUpload.dao.findById(id);
  }
  
  public Long save(String str)
  {
    try
    {
      Long id = IdGen.getId();
      str = str.replaceAll("[\\s*\t\n\r]", "");
      Base64Entity entity = Base64Util.parse(str);
      String name = System.currentTimeMillis() + MD5Utils.salt() + "." + entity.getExt();
      String path = FileUtil.initFilePath(name);
      FileUpload fu = new FileUpload();
      fu.setCreateTime(System.currentTimeMillis());
      fu.setFilePath(path);
      fu.setFileSize(Long.valueOf(entity.getContent().length));
      fu.setId(id);
      fu.setFileType(entity.getType());
      fu.setFileName(name);
      fu.setFileExt(entity.getExt());
      FileUtil.write(new ByteArrayInputStream(entity.getContent()), new FileOutputStream(path));
      entity = null;
      fu.save();
      return id;
    }
    catch (Exception e)
    {
      e.printStackTrace();
      throw new XgsException(Constants.error.getInt("file.writeerror"));
    }
  }
  
  @Before({Tx.class})
  public List<Long> saveFiles(List<UploadFile> ups)
  {
    List<Long> strs = new ArrayList<>();
    for (UploadFile up : ups) {
      strs.add(save(up));
    }
    return strs;
  }
  
  public Long save(UploadFile up, Long id)
  {
    try
    {
      File file = up.getFile();
      String path = FileUtil.initFilePath(file.getName());
      FileUpload fu = new FileUpload();
      fu.setCreateTime(Long.valueOf(System.currentTimeMillis()));
      fu.setFilePath(path);
      fu.setFileSize(Long.valueOf(file.length()));
      fu.setId(id);
      String ext = FileUtil.getExtension(file.getName());
      fu.setFileType(up.getContentType());
      fu.setFileName(file.getName());
      fu.setFileExt(ext);
      FileUtil.write(new FileInputStream(file), new FileOutputStream(path));
      file.delete();
      fu.save();
      return id;
    }
    catch (Exception e)
    {
      e.printStackTrace();
      throw new XgsException(Constants.error.getInt("file.writeerror"));
    }
  }
  
  public Long save(UploadFile up)
  {
    try
    {
      Long id = IdGen.getId();
      save(up, id);
      return id;
    }
    catch (Exception e)
    {
      e.printStackTrace();
      throw new XgsException(Constants.error.getInt("file.writeerror"));
    }
  }
}
