package xgs.net.util;

import xgs.net.common.Constants;
import xgs.net.utils.StrUtils;

import javax.servlet.ServletOutputStream;
import javax.swing.filechooser.FileSystemView;
import java.io.*;
import java.nio.channels.FileChannel;
import java.text.CollationKey;
import java.text.Collator;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;

public class FileUtil
{
    public static String FILEDIR;

    public static void sortFiles(final File[] files) throws Exception {
        sortFiles(files, 0, false);
    }

    public static void sortFiles(final File[] files, final int type, final boolean desc) throws Exception {
        Arrays.sort(files, new Comparator<File>() {
            Collator collator = Collator.getInstance();

            @Override
            public int compare(final File f1, final File f2) {
                switch (type) {
                    case 1: {
                        final Long l1 = f1.isDirectory() ? -1L : f1.length();
                        final Long l2 = f2.isDirectory() ? -1L : f2.length();
                        if (desc) {
                            return l2.compareTo(l1);
                        }
                        return l1.compareTo(l2);
                    }
                    case 2: {
                        final CollationKey t1 = this.collator.getCollationKey(f1.isDirectory() ? "0" : ("1" + FileUtil.getFileType(f1).toLowerCase()));
                        final CollationKey t2 = this.collator.getCollationKey(f2.isDirectory() ? "0" : ("1" + FileUtil.getFileType(f2).toLowerCase()));
                        if (desc) {
                            return t2.compareTo(t1);
                        }
                        return t1.compareTo(t2);
                    }
                    case 3: {
                        Long d1 = f1.lastModified();
                        Long d2 = f2.lastModified();
                        final boolean b1 = f1.isDirectory();
                        final boolean b2 = f2.isDirectory();
                        if (b1 && !b2) {
                            d1 = Long.MIN_VALUE;
                        }
                        if (b2 && !b1) {
                            d2 = Long.MIN_VALUE;
                        }
                        if (desc) {
                            return d2.compareTo(d1);
                        }
                        return d1.compareTo(d2);
                    }
                    default: {
                        final CollationKey k1 = this.collator.getCollationKey((f1.isDirectory() ? 0 : 1) + f1.getName().toLowerCase());
                        final CollationKey k2 = this.collator.getCollationKey((f2.isDirectory() ? 0 : 1) + f2.getName().toLowerCase());
                        if (desc) {
                            return k2.compareTo(k1);
                        }
                        return k1.compareTo(k2);
                    }
                }
            }
        });
    }

    public static boolean deleteFolder(final File folder) {
        final File[] listFiles;
        final File[] files = listFiles = folder.listFiles();
        for (final File file : listFiles) {
            if (file.isDirectory()) {
                deleteFolder(file);
            }
            else if (!file.delete()) {
                return false;
            }
        }
        return folder.delete();
    }

    public static File getUniqueFile(File file) {
        if (file.exists()) {
            final String path = file.getAbsolutePath();
            final String dir = extractDir(path);
            String name = extractFilename(path);
            int i = name.lastIndexOf(46);
            String ext;
            if (i > -1 && i < name.length() - 1) {
                ext = name.substring(i);
                name = name.substring(0, i);
            }
            else {
                ext = "";
            }
            i = 1;
            do {
                file = new File(dir, name + i + ext);
                ++i;
            } while (file.exists());
        }
        return file;
    }

    public static File[] getFiles(final String path) {
        final File f = new File(path);
        if (f != null && f.isDirectory()) {
            return f.listFiles();
        }
        return null;
    }

    public static void copyFile(final File source, File dest, final boolean autoRename, final boolean isCut) throws Exception {
        boolean exists = dest.exists();
        if (autoRename && exists) {
            dest = getUniqueFile(dest);
            exists = false;
        }
        if (!exists) {
            dest.createNewFile();
        }
        FileChannel in = null;
        FileChannel out = null;
        try {
            in = new FileInputStream(source).getChannel();
            out = new FileOutputStream(dest).getChannel();
            in.transferTo(0L, in.size(), out);
        }
        finally {
            if (in != null) {
                try {
                    in.close();
                }
                catch (Throwable t) {}
            }
            if (out != null) {
                out.close();
            }
        }
        dest.setLastModified(source.lastModified());
        if (isCut) {
            source.delete();
        }
    }

    public static void copyFolder(final File source, File dest, final boolean autoRename, final boolean isCut) throws Exception {
        boolean exists = dest.exists();
        if (autoRename && exists) {
            dest = getUniqueFile(dest);
            exists = false;
        }
        if (!exists) {
            dest.mkdirs();
        }
        final File destDir = dest;
        final File[] listFiles;
        final File[] files = listFiles = source.listFiles();
        for (final File file : listFiles) {
            final String name = file.getName();
            final File destFile = new File(destDir, name);
            if (file.isFile()) {
                copyFile(file, destFile, autoRename, isCut);
            }
            else {
                copyFolder(file, destFile, autoRename, isCut);
            }
        }
        if (isCut) {
            source.delete();
        }
    }

    public static String readText(final File file, final String charset) throws Exception {
        final FileInputStream fs = new FileInputStream(file);
        try {
            final int size = (int)file.length();
            final byte[] bs = new byte[size];
            fs.read(bs);
            if (StrUtils.isBlank(charset)) {
                return new String(bs);
            }
            return new String(bs, charset);
        }
        finally {
            fs.close();
        }
    }

    public static String readUtfText(final File file) throws Exception {
        return readText(file, "utf-8");
    }

    public static void writeText(final File file, final String content, final String charset) throws Exception {
        if (!file.exists()) {
            file.createNewFile();
        }
        final FileOutputStream os = new FileOutputStream(file);
        try {
            if (StrUtils.isBlank(charset)) {
                os.write(content.getBytes());
            }
            else {
                os.write(content.getBytes(charset));
            }
        }
        finally {
            os.close();
        }
    }

    public static void writeUtfText(final File file, final String content) throws Exception {
        writeText(file, content, "utf-8");
    }

    public static String extractFileExt(final String fileName) {
        if (!StrUtils.isBlank(fileName)) {
            final int i = fileName.lastIndexOf(46);
            if (i != -1) {
                return fileName.substring(i + 1);
            }
        }
        return "";
    }

    public static String getFileType(final File file) {
        String type;
        try {
            type = FileSystemView.getFileSystemView().getSystemTypeDescription(file);
        }
        catch (Throwable e) {
            type = null;
        }
        if (StrUtils.isBlank(type)) {
            return extractFileExt(file.getName());
        }
        return type;
    }

    public static String extractFilename(final String fileName) {
        if (StrUtils.isBlank(fileName)) {
            return "";
        }
        final int p = Math.max(fileName.lastIndexOf(47), fileName.lastIndexOf(92));
        if (p == -1) {
            return fileName;
        }
        return fileName.substring(p + 1);
    }

    public static String extractFilenameNoExt(final String fileName) {
        final String s = extractFilename(fileName);
        final int i = s.lastIndexOf(46);
        if (i != -1) {
            return s.substring(0, i);
        }
        return s;
    }

    public static String getPath(final String path) {
        return path.replace("\\", "/");
    }

    public static String getPath(final File file) {
        return getPath(file.getAbsolutePath());
    }

    public static String extractDir(String fileName) {
        if (StrUtils.isBlank(fileName)) {
            return "";
        }
        fileName = getPath(fileName);
        final int pos = fileName.lastIndexOf(47);
        if (pos == -1) {
            return fileName;
        }
        return fileName.substring(0, pos);
    }

    public static boolean isAncestor(final File parent, final File child) {
        final String p = getPath(parent) + "/";
        final String c = getPath(child) + "/";
        return c.length() > p.length() && c.substring(0, p.length()).equalsIgnoreCase(p);
    }

    public static boolean hasSubFile(final File file, final boolean isDir) {
        final File[] files = file.listFiles();
        if (files == null) {
            return false;
        }
        for (final File f : files) {
            if (!isDir || f.isDirectory()) {
                return true;
            }
        }
        return false;
    }

    public static void write(final InputStream in, final OutputStream out) throws IOException {
        try {
            final byte[] buffer = new byte[1024000];
            int bytesRead = -1;
            while ((bytesRead = in.read(buffer)) != -1) {
                out.write(buffer, 0, bytesRead);
            }
            out.flush();
        }
        finally {
            try {
                in.close();
            }
            catch (IOException ex) {}
            try {
                out.close();
            }
            catch (IOException ex2) {}
        }
    }

    public static void download(final String downloadfFileName, final ServletOutputStream out) {
        try {
            final FileInputStream in = new FileInputStream(new File(FileUtil.FILEDIR + "/" + downloadfFileName));
            write(in, out);
        }
        catch (FileNotFoundException e3) {
            try {
                final FileInputStream in2 = new FileInputStream(new File(FileUtil.FILEDIR + "/" + new String(downloadfFileName.getBytes("iso-8859-1"), "utf-8")));
                write(in2, out);
            }
            catch (IOException e1) {
                e1.printStackTrace();
            }
        }
        catch (IOException e2) {
            e2.printStackTrace();
        }
    }

    public static String initFilePath(final String name) {
        final String dir = getFileDir(name) + "";
        final File file = new File(FileUtil.FILEDIR + dir);
        if (!file.exists()) {
            file.mkdir();
        }
        final Long num = new Date().getTime();
        final Double d = Math.random() * num;
        return (file.getPath() + "/" + num + d + "_" + name).replaceAll(" ", "-");
    }

    private static int getFileDir(final String name) {
        return name.hashCode() & 0xF;
    }

    public static String getFilesInfo(final File[] files, final boolean check) {
        final StringBuilder buf = new StringBuilder();
        boolean isFirst = true;
        buf.append('[');
        for (final File file : files) {
            final boolean isDir = file.isDirectory();
            String name = file.getName();
            final String dir = getPath(file);
            if (StrUtils.isBlank(name)) {
                name = extractDir(dir);
            }
            if (isFirst) {
                isFirst = false;
            }
            else {
                buf.append(',');
            }
            name = file.getName();
            buf.append("{text:\"");
            buf.append(name);
            buf.append("\",dir:\"");
            buf.append(dir);
            buf.append("\"");
            buf.append(",isDir:");
            buf.append(isDir);
            if (!hasSubFile(file, false)) {
                if (isDir) {
                    buf.append(",children:[]");
                }
                else {
                    if (check) {
                        buf.append(",checked:false");
                    }
                    buf.append(",leaf:true,iconCls:\"default_icon\"");
                }
            }
            buf.append('}');
        }
        buf.append(']');
        return buf.toString();
    }

    public static final String getExtension(final String fileName) {
        final String[] names = fileName.split("\\.");
        if (names.length == 1) {
            return "";
        }
        return names[names.length - 1];
    }

    static {
        FileUtil.FILEDIR = Constants.uploadDir + File.separator + "uploadfile/files";
    }
}
