package xgs.net.util;

import net.coobird.thumbnailator.Thumbnails;
import net.coobird.thumbnailator.geometry.Positions;
import net.coobird.thumbnailator.tasks.io.FileImageSource;
import xgs.net.common.Constants;
import xgs.net.exception.XgsException;

import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.stream.ImageInputStream;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Iterator;

public class ImageUtil {
    private static String getImageFormatName(File file)
        throws IOException {
        String formatName = null;

        ImageInputStream iis = ImageIO.createImageInputStream(file);
        Iterator<ImageReader> imageReader = ImageIO.getImageReaders(iis);
        while (imageReader.hasNext()) {
            ImageReader reader = imageReader.next();
            formatName = reader.getFormatName();
        }
        return formatName;
    }

    private static String getFileChildPath(File file) {
        String name = file.getAbsolutePath();
        return name.substring(0, name.lastIndexOf('.'));
    }

    private static String regionFileName(int width, int height, File file)
        throws IOException {
        return width + "-" + height + "." + getImageFormatName(file).toLowerCase();
    }

    public static File scaleSize(File src, Integer height, Integer width)
        throws IOException {
        if (!src.exists()) {
            throw new XgsException(Constants.error.getInt("file.notExist"));
        }
        File distPath = new File(getFileChildPath(src));
        if (!distPath.exists()) {
            distPath.mkdirs();
        }
        String file_name = regionFileName(width, height, src);
        File dist = new File(distPath, file_name);
        if (!dist.exists()) {
            Thumbnails.Builder<File> builder = Thumbnails.of(src);
            builder.size(width, height).outputQuality(1.0D).toFile(dist);
        }
        return dist;
    }

    public static File scale(File src, Integer height, Integer width)
        throws IOException {
        if (!src.exists()) {
            throw new XgsException(Constants.error.getInt("file.notExist"));
        }
        File distPath = new File(getFileChildPath(src));
        if (!distPath.exists()) {
            distPath.mkdirs();
        }
        FileImageSource fileImageSource = new FileImageSource(src);
        BufferedImage image = fileImageSource.read();
        int _width = width;
        int _height = height;
        if (image.getHeight() > image.getWidth() || height == null) {
            double s = image.getWidth() / Double.valueOf(image.getHeight());
            _width = Double.valueOf(width / s).intValue();
//        _height = Double.valueOf(_height * s).intValue();
        } else {
            double s = image.getHeight() / Double.valueOf(image.getWidth());
            _height = Double.valueOf(height / s).intValue();
//        _width = Double.valueOf(_width * s).intValue();
        }
        String file_name = regionFileName(_width, _height, src);
        File dist = new File(distPath, file_name);
        if (!dist.exists()) {
            Thumbnails.Builder<File> builder = Thumbnails.of(src);
            if (_height > _width || height == null) {
                builder.height(height).outputQuality(1.0D).toFile(dist);
            } else {
                builder.width(width).outputQuality(1.0D).toFile(dist);
            }
        }
        return dist;
    }

    public static File regionCenter(File src, Integer height, Integer width)
        throws IOException {
        File temp = scale(src, height, width);
        File distPath = new File(getFileChildPath(src));
        String file_name = regionFileName(width, height, temp);
        File dist = new File(distPath, file_name);
        if (!dist.exists()) {
            Thumbnails.Builder<File> builder = Thumbnails.of(temp);
            FileImageSource fileImageSource = new FileImageSource(temp);
            BufferedImage image = fileImageSource.read();
            builder.size(image.getWidth(), image.getHeight()).outputQuality(1.0D).sourceRegion(Positions.CENTER, width, height).toFile(dist);
        }
        return dist;
    }
}
