package xgs.net.plugin;

import com.jfinal.aop.Enhancer;
import com.jfinal.aop.Interceptor;
import com.jfinal.config.Routes;
import com.jfinal.core.Controller;
import com.jfinal.kit.StrKit;
import com.jfinal.plugin.IPlugin;
import com.jfinal.plugin.activerecord.Model;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import xgs.net.annotation.ControllerKey;
import xgs.net.annotation.Entity;
import xgs.net.annotation.Service;
import xgs.net.plugin.ioc.InjectUtils;
import xgs.net.plugin.ioc.IocKit;
import xgs.net.searcher.ClassSearcher;

import java.util.Collection;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

public class IocPlugin implements IPlugin
{
    private String[] pkgs;
    private Routes routes;
    private Interceptor[] interceptors;
    private Logger logger;
    private static final ConcurrentMap<String, Object> iocBeanMap;
    
    public IocPlugin(final String[] pkgs, final Interceptor... interceptors) {
        this.routes = new Routes() {
            public void config() {
            }
        };
        this.logger = LoggerFactory.getLogger(this.getClass());
        this.pkgs = pkgs;
        this.interceptors = interceptors;
    }
    
    public Routes getRoutes() {
        return this.routes;
    }
    
    public boolean start() {
        this.logger.info("loading annotaion");
        for (final Interceptor inter : this.interceptors) {
            this.routes.addInterceptor(inter);
        }
        final Set<Class<?>> classes = ClassSearcher.getClasses(this.pkgs, new Class[] { ControllerKey.class, Service.class, Entity.class });
        for (final Class<?> clazz : classes) {
            final String beanName = clazz.getName();
            if (Controller.class.isAssignableFrom(clazz)) {
                this.initController(clazz);
            } else if (clazz.getAnnotation(Entity.class) != null) {
                try {
                    final Model<?> model = (Model<?>)clazz.newInstance();
                    IocPlugin.iocBeanMap.put(beanName, model);
                } catch (Exception ex) {}
            } else {
                if (clazz.getAnnotation(Service.class) == null) {
                    continue;
                }
                final Object enhanceBean = Enhancer.enhance((Class)clazz, this.interceptors);
                if (IocPlugin.iocBeanMap.containsKey(beanName)) {
                    this.logger.warn("bean:" + beanName + " reloading!");
                }
                IocPlugin.iocBeanMap.put(beanName, enhanceBean);
            }
        }
        IocKit.init(IocPlugin.iocBeanMap);
        final Collection<Object> beanColl = IocPlugin.iocBeanMap.values();
        for (final Object object : beanColl) {
            Class<?> superclass;
            if (object.getClass().getAnnotation(Entity.class) != null) {
                superclass = object.getClass();
            }
            else {
                superclass = object.getClass().getSuperclass();
            }
            InjectUtils.inject(superclass, object);
        }
        this.logger.info("net.xgs.annotation loaded ");
        return true;
    }
    
    public boolean stop() {
        IocPlugin.iocBeanMap.clear();
        return true;
    }
    
    private void initController(final Class<?> clazz) {
        final ControllerKey controller = clazz.getAnnotation(ControllerKey.class);
        String viewPath = controller.viewPath();
        if (StrKit.isBlank(viewPath)) {
            viewPath = controller.value();
        }
        this.routes.add(controller.value(), (Class)clazz, viewPath);
    }
    
    static {
        iocBeanMap = new ConcurrentHashMap<String, Object>();
    }
}
