package xgs.net.plugin.ioc;

import com.jfinal.log.Log;
import xgs.net.annotation.Entity;
import xgs.net.annotation.Inject;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class InjectUtils
{
    private static Log log;
    
    public static void inject(final Object target) {
        final Class<?> clazz = target.getClass();
        inject(clazz, target);
    }
    
    public static void inject(final Class<?> clazz, final Object target) {
        List<Field> fields = IocKit.getFields(clazz);
        final List<Field> fieldNows = new ArrayList<Field>();
        boolean isNow = false;
        if (fields == null || fields.isEmpty()) {
            isNow = true;
            fields = Arrays.asList(clazz.getDeclaredFields());
        }
        for (final Field field : fields) {
            if (null != field) {
                if (!field.isAnnotationPresent(Inject.class)) {
                    continue;
                }
                final int modifiers = field.getModifiers();
                if (Modifier.isFinal(modifiers)) {
                    throw new RuntimeException("Class: " + clazz + " field: " + field.getName() + " is final!");
                }
                if (Modifier.isStatic(modifiers)) {
                    InjectUtils.log.warn("Class " + clazz + " field " + field.getName() + "is static!");
                }
                if (isNow) {
                    fieldNows.add(field);
                }
                final Object value = IocKit.getBean(field.getType());
                if (value == null) {
                    throw new RuntimeException("not has bean:" + field.getType());
                }
                if (!field.isAccessible()) {
                    field.setAccessible(true);
                }
                try {
                    field.set(target, value);
                }
                catch (IllegalArgumentException e) {
                    InjectUtils.log.error(e.getMessage(), (Throwable)e);
                    throw new RuntimeException(e);
                }
                catch (IllegalAccessException e2) {
                    InjectUtils.log.error(e2.getMessage(), (Throwable)e2);
                    throw new RuntimeException(e2);
                }
            }
        }
        genericInjection(clazz, target);
        IocKit.saveBean(clazz, fieldNows);
    }
    
    private static void genericInjection(Class<?> clazz, final Object target) {
        try {
            if (clazz.getAnnotation(Entity.class) != null) {
                return;
            }
            final Type type = clazz.getGenericSuperclass();
            if (type == null || type == Object.class) {
                return;
            }
            if (!(type instanceof ParameterizedType)) {
                final List<Field> fields = Arrays.asList(clazz.getSuperclass().getDeclaredFields());
                for (final Field field : fields) {
                    if (null != field) {
                        if (!field.isAnnotationPresent(Inject.class)) {
                            continue;
                        }
                        field.setAccessible(true);
                        field.set(target, IocKit.getBean(field.getType()));
                    }
                }
                return;
            }
            final ParameterizedType o = (ParameterizedType)type;
            final Type[] parameterizedType = o.getActualTypeArguments();
            do {
                final List<Field> fields2 = Arrays.asList(clazz.getSuperclass().getDeclaredFields());
                for (final Field field2 : fields2) {
                    field2.setAccessible(true);
                    for (final Type generic : parameterizedType) {
                        final Class<?> claz = (Class<?>)generic;
                        if (field2.getType().isAssignableFrom(claz)) {
                            field2.set(target, IocKit.getBean(claz));
                            break;
                        }
                    }
                }
                clazz = clazz.getSuperclass();
            } while (clazz.getSuperclass() != null && clazz.getSuperclass() != Object.class);
        }
        catch (IllegalAccessException e) {
            e.printStackTrace();
        }
    }
    
    static {
        InjectUtils.log = Log.getLog((Class)InjectUtils.class);
    }
}
