package xgs.net.plugin.ioc;

import java.lang.reflect.Field;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

public class IocKit
{
    private static ConcurrentMap<String, IocObject> beanMap;
    
    public static void init(final ConcurrentMap<String, Object> beanMap) {
        IocKit.beanMap = new ConcurrentHashMap<String, IocObject>();
        if (beanMap == null || beanMap.isEmpty()) {
            return;
        }
        for (final String key : beanMap.keySet()) {
            IocKit.beanMap.put(key, new IocObject(beanMap.get(key), null));
        }
    }
    
    public static ConcurrentMap<String, IocObject> getBeanMap() {
        return IocKit.beanMap;
    }
    
    public static <T> T getBean(final Class<?> clazz) {
        final String beanName = clazz.getName();
        final IocObject target = IocKit.beanMap.get(beanName);
        if (target == null) {
            return null;
        }
        return (T)target.getTarget();
    }
    
    public static <T> T getFields(final Class<?> clazz) {
        final String beanName = clazz.getName();
        final IocObject target = IocKit.beanMap.get(beanName);
        if (target == null) {
            return null;
        }
        return (T)target.getFields();
    }
    
    public static void saveBean(final Class<?> clazz, final List<Field> fields) {
        final String beanName = clazz.getName();
        if (IocKit.beanMap.get(beanName) == null) {
            return;
        }
        IocKit.beanMap.get(beanName).setFields(fields);
    }
}
