package xgs.net.plugin;

import com.google.common.collect.Sets;
import com.jfinal.aop.Enhancer;
import com.jfinal.plugin.IPlugin;
import xgs.net.annotation.Act;
import xgs.net.annotation.Ctl;
import xgs.net.entity.ActEntity;
import xgs.net.entity.CtlEntity;
import xgs.net.entity.ParamEntity;
import xgs.net.plugin.ioc.InjectUtils;
import xgs.net.searcher.ClassSearcher;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.Set;

public class MvcPlugin implements IPlugin
{
    private String[] pkgs;
    
    public MvcPlugin(final String... pkgs) {
        this.pkgs = pkgs;
    }
    
    public boolean start() {
        final Set<Class<?>> classes = (Set<Class<?>>)ClassSearcher.getClasses(this.pkgs, new Class[] { Ctl.class });
        for (final Class<?> clazz : classes) {
            if (clazz.getAnnotation(Ctl.class) != null) {
                final Ctl ctl = clazz.getAnnotation(Ctl.class);
                final CtlEntity ce = new CtlEntity();
                ce.setClazz(clazz);
                final Object obj = Enhancer.enhance((Class)clazz);
                ce.setObj(obj);
                ce.setCtl(ctl.value());
                initAct(clazz, obj, ce);
                MvcBeans.setCtl(ctl, ce);
            }
        }
        return true;
    }
    
    public boolean stop() {
        return true;
    }
    
    public static void initAct(final Class<?> clazz, final Object target, final CtlEntity ce) {
        InjectUtils.inject((Class)clazz, target);
        final Set<Method> methods = loopMethods(clazz);
        if (methods == null || methods.size() == 0) {
            return;
        }
        for (final Method method : methods) {
            if (null != method) {
                if (!method.isAnnotationPresent(Act.class)) {
                    continue;
                }
                final Act act = method.getAnnotation(Act.class);
                if (ce.getActs().containsKey(act.value())) {
                    throw new RuntimeException("multi action values:" + act.value());
                }
                final Annotation[][] ass = method.getParameterAnnotations();
                final Class<?>[] clazzs = method.getParameterTypes();
                final ActEntity ae = new ActEntity();
                if (clazzs != null && clazzs.length != 0) {
                    int i = 0;
                    for (final Class<?> temp : clazzs) {
                        final ParamEntity pe = new ParamEntity();
                        pe.setType(temp);
                        if (ass[i].length != 0) {
                            pe.setAnnotation(ass[i][0]);
                        }
                        ae.getParameters().add(pe);
                        ++i;
                    }
                }
                ae.setAct(act.value());
                ae.setMethod(method);
                ce.getActs().put(act.value(), ae);
            }
        }
    }
    
    private static Set<Method> loopMethods(Class<?> clazz) {
        final Set<Method> methods = Sets.newHashSet();
        do {
            final Method[] declaredMethods;
            final Method[] temp = declaredMethods = clazz.getDeclaredMethods();
            for (final Method t : declaredMethods) {
                if (!methods.contains(t)) {
                    methods.add(t);
                }
            }
            clazz = clazz.getSuperclass();
        } while (clazz == Object.class);
        return methods;
    }
}
