package xgs.net.searcher;

import com.jfinal.log.Log;
import xgs.net.utils.ClassUtils;

import java.io.IOException;
import java.lang.annotation.Annotation;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

public class ClassSearcher
{
    private static final Log log;
    
    public static Set<Class<?>> getClasses(final List<String> packageNames, final Class<? extends Annotation> annotation) {
        final String[] pkgs = packageNames.toArray(new String[packageNames.size()]);
        return getClasses(pkgs, annotation);
    }
    
    public static Set<Class<?>> getClasses(final List<String> packageNames, final Class<? extends Annotation>[] annotations) {
        final String[] pkgs = packageNames.toArray(new String[packageNames.size()]);
        return getClasses(pkgs, annotations);
    }
    
    public static Set<Class<?>> getClasses(final String[] packageNames, final Class<? extends Annotation>... annotations) {
        final AnnotationReader reader = new AnnotationReader();
        for (final Class<? extends Annotation> annotation : annotations) {
            reader.addAnnotation(annotation);
        }
        final Set<Class<?>> classes = new LinkedHashSet<Class<?>>();
        final FileSearcher finder = new FileSearcher() {
            public void visitFileEntry(final FileEntry file) {
                if (file.isJavaClass()) {
                    try {
                        if (reader.isAnnotationed(file.getInputStream())) {
                            this.addClass(file.getQualifiedJavaName());
                        }
                    }
                    catch (IOException e) {
                        throw new RuntimeException(e);
                    }
                }
            }
            
            private void addClass(final String qualifiedClassName) {
                try {
                    final Class<?> klass = ClassUtils.loadClass(qualifiedClassName);
                    classes.add(klass);
                }
                catch (Throwable e) {
                    ClassSearcher.log.warn("Class load error.", e);
                }
            }
        };
        finder.lookupClasspath(packageNames);
        return classes;
    }
    
    static {
        log = Log.getLog((Class)ClassSearcher.class);
    }
}
