package xgs.net.utils;

import com.jfinal.plugin.activerecord.CPI;
import com.jfinal.plugin.activerecord.Model;
import com.jfinal.plugin.activerecord.Record;
import net.sf.cglib.beans.BeanCopier;
import net.sf.cglib.beans.BeanGenerator;
import net.sf.cglib.beans.BeanMap;
import net.sf.cglib.core.Converter;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

public class BeanUtils
{
    private static final ConcurrentMap<Class<?>, PropertyDescriptor[]> classPropCache;
    
    public static Object generator(final Object superBean, final BeanProperty... props) {
        final Class<?> superclass = superBean.getClass();
        final Object genBean = generator(superclass, props);
        copy(superBean, genBean);
        return genBean;
    }
    
    public static Object generator(final Class<?> superclass, final BeanProperty... props) {
        final BeanGenerator generator = new BeanGenerator();
        generator.setSuperclass((Class)superclass);
        for (final BeanProperty prop : props) {
            generator.addProperty(prop.getName(), (Class)prop.getType());
        }
        return generator.create();
    }
    
    public static <T> T copy(final Object src, final Class<T> clazz) {
        final BeanCopier copier = BeanCopier.create((Class)src.getClass(), (Class)clazz, false);
        final T to = ClassUtils.newInstance(clazz);
        copier.copy(src, (Object)to, (Converter)null);
        return to;
    }
    
    public static <T> T copy(final Object src, final Class<T> clazz, final Converter converter) {
        final BeanCopier copier = BeanCopier.create((Class)src.getClass(), (Class)clazz, true);
        final T to = ClassUtils.newInstance(clazz);
        copier.copy(src, (Object)to, converter);
        return to;
    }
    
    public static void copy(final Object src, final Object dist) {
        final BeanCopier copier = BeanCopier.create((Class)src.getClass(), (Class)dist.getClass(), false);
        copier.copy(src, dist, (Converter)null);
    }
    
    public static void copy(final Object src, final Object dist, final Converter converter) {
        final BeanCopier copier = BeanCopier.create((Class)src.getClass(), (Class)dist.getClass(), true);
        copier.copy(src, dist, converter);
    }
    
    public static <T> T toBean(final Model<?> model, final Class<T> valueType) {
        return toBean(CPI.getAttrs((Model)model), valueType);
    }
    
    public static <T> T toBean(final Record record, final Class<T> valueType) {
        return toBean(record.getColumns(), valueType);
    }
    
    public static <T> T toBean(final Map<String, Object> beanMap, final Class<T> valueType) {
        final T bean = ClassUtils.newInstance(valueType);
        final PropertyDescriptor[] propertyDescriptors;
        final PropertyDescriptor[] beanPds = propertyDescriptors = getPropertyDescriptors(bean);
        for (final PropertyDescriptor propDescriptor : propertyDescriptors) {
            final String propName = propDescriptor.getName();
            if (!propName.equals("class")) {
                if (beanMap.containsKey(propName)) {
                    final Method writeMethod = propDescriptor.getWriteMethod();
                    if (null != writeMethod) {
                        final Object value = beanMap.get(propName);
                        if (!writeMethod.isAccessible()) {
                            writeMethod.setAccessible(true);
                        }
                        try {
                            writeMethod.invoke(bean, value);
                        }
                        catch (Throwable e) {
                            throw new RuntimeException("Could not set property '" + propName + "' to bean", e);
                        }
                    }
                }
            }
        }
        return bean;
    }
    
    public static Map toMap(final Object src) {
        return (Map)BeanMap.create(src);
    }
    
    public static void copy(final Model<?> src, final Model<?> dist) {
        dist._setAttrs(CPI.getAttrs((Model)src));
    }
    
    public static void copy(final Model<?> src, final Record dist) {
        dist.setColumns((Model)src);
    }
    
    public static void copy(final Record src, final Model<?> dist) {
        dist._setAttrs(src.getColumns());
    }
    
    public static void copy(final Object src, final Model<?> dist) {
        final Map<String, Object> attrs = (Map<String, Object>)toMap(src);
        dist._setAttrs((Map)attrs);
    }
    
    public static void copy(final Object src, final Record dist) {
        final Map<String, Object> columns = (Map<String, Object>)toMap(src);
        dist.setColumns((Map)columns);
    }
    
    private static PropertyDescriptor[] getPropertyDescriptors(final Object bean) {
        final Class<?> beanClass = bean.getClass();
        PropertyDescriptor[] cachePds = BeanUtils.classPropCache.get(beanClass);
        if (null != cachePds) {
            return cachePds;
        }
        try {
            final BeanInfo beanInfo = Introspector.getBeanInfo(beanClass);
            cachePds = beanInfo.getPropertyDescriptors();
            BeanUtils.classPropCache.put(beanClass, cachePds);
            return cachePds;
        }
        catch (IntrospectionException e) {
            throw new RuntimeException(e);
        }
    }
    
    public static Object getProperty(final Object bean, final String propertyName) {
        final PropertyDescriptor[] propertyDescriptors;
        final PropertyDescriptor[] beanPds = propertyDescriptors = getPropertyDescriptors(bean);
        for (final PropertyDescriptor propertyDescriptor : propertyDescriptors) {
            if (propertyDescriptor.getName().equals(propertyName)) {
                final Method readMethod = propertyDescriptor.getReadMethod();
                if (null != readMethod) {
                    if (!readMethod.isAccessible()) {
                        readMethod.setAccessible(true);
                    }
                    try {
                        return readMethod.invoke(bean, new Object[0]);
                    }
                    catch (Throwable ex) {
                        throw new RuntimeException("Could not read property '" + propertyName + "' from bean", ex);
                    }
                }
            }
        }
        return null;
    }
    
    public static void setProperty(final Object bean, final String propertyName, final Object value) {
        final PropertyDescriptor[] propertyDescriptors;
        final PropertyDescriptor[] beanPds = propertyDescriptors = getPropertyDescriptors(bean);
        for (final PropertyDescriptor propertyDescriptor : propertyDescriptors) {
            if (propertyDescriptor.getName().equals(propertyName)) {
                final Method writeMethod = propertyDescriptor.getWriteMethod();
                if (null != writeMethod) {
                    if (!writeMethod.isAccessible()) {
                        writeMethod.setAccessible(true);
                    }
                    try {
                        writeMethod.invoke(bean, value);
                    }
                    catch (Throwable ex) {
                        throw new RuntimeException("Could not set property '" + propertyName + "' to bean", ex);
                    }
                }
            }
        }
    }
    
    public static void main(final String[] args) {
        final Object entity = ClassUtils.newInstance("net.xgs.utils.Entity");
        setProperty(entity, "name", "zhangsan");
        setProperty(entity, "code", "zs");
        setProperty(entity, "content", "child");
        System.out.println(entity.toString());
    }
    
    static {
        classPropCache = new ConcurrentHashMap<Class<?>, PropertyDescriptor[]>(64);
    }
}
