package xgs.net.utils;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.*;

public abstract class ClassUtils
{
    private static final String EXT_CLASS_LOADER_NAME = "sun.misc.Launcher$ExtClassLoader";
    
    public static boolean isPresent(final String className, final ClassLoader classLoader) {
        try {
            Class.forName(className, true, classLoader);
            return true;
        }
        catch (Throwable ex) {
            return false;
        }
    }
    
    public static <T> T newInstance(final Class<?> clazz) {
        try {
            return (T)clazz.newInstance();
        }
        catch (InstantiationException e) {
            throw new RuntimeException(e);
        }
        catch (IllegalAccessException e2) {
            throw new RuntimeException(e2);
        }
    }
    
    public static <T> T newInstance(final String clazzStr) {
        final Class<?> clazz = loadClass(clazzStr);
        return newInstance(clazz);
    }
    
    public static ClassLoader getContextClassLoader() {
        ClassLoader classLoader = null;
        try {
            classLoader = Thread.currentThread().getContextClassLoader();
        }
        catch (Throwable t) {}
        return classLoader;
    }
    
    public static ClassLoader getClassLoader() {
        ClassLoader classLoader = getContextClassLoader();
        if (classLoader == null) {
            classLoader = ClassUtils.class.getClassLoader();
            if (classLoader == null) {
                try {
                    classLoader = ClassLoader.getSystemClassLoader();
                }
                catch (Throwable t) {}
            }
        }
        return classLoader;
    }
    
    public static Class<?> loadClass(final String className) {
        if (className == null) {
            return null;
        }
        final ClassLoader classLoader = getClassLoader();
        try {
            return (classLoader != null) ? classLoader.loadClass(className) : Class.forName(className);
        }
        catch (ClassNotFoundException e) {
            final int lastDotIndex = className.lastIndexOf(46);
            if (lastDotIndex != -1) {
                final String innerClassName = className.substring(0, lastDotIndex) + '$' + className.substring(lastDotIndex + 1);
                try {
                    return (classLoader != null) ? classLoader.loadClass(innerClassName) : Class.forName(innerClassName);
                }
                catch (ClassNotFoundException ex) {}
            }
            return null;
        }
    }
    
    private static Collection<URL> getClasspathURLs(final ClassLoader classLoader) {
        final Collection<URL> urls = new LinkedHashSet<URL>(32);
        for (ClassLoader loader = classLoader; loader != null; loader = loader.getParent()) {
            final String klassName = loader.getClass().getName();
            if ("sun.misc.Launcher$ExtClassLoader".equals(klassName)) {
                break;
            }
            if (loader instanceof URLClassLoader) {
                for (final URL url : ((URLClassLoader)loader).getURLs()) {
                    urls.add(url);
                }
            }
            else if (klassName.startsWith("weblogic.utils.classloaders.")) {
                try {
                    final Method method = loader.getClass().getMethod("getClassPath", (Class<?>[])new Class[0]);
                    final Object result = method.invoke(loader, new Object[0]);
                    if (result != null) {
                        final String[] split;
                        final String[] paths = split = StrUtils.split(result.toString(), File.pathSeparatorChar);
                        for (final String path : split) {
                            urls.add(URLUtils.fromFile(path));
                        }
                    }
                }
                catch (NoSuchMethodException ex) {}
                catch (Exception e) {
                    throw new RuntimeException(e);
                }
            }
        }
        final String classpath = System.getProperty("java.class.path");
        if (classpath.length() > 1) {
            final String[] split2;
            final String[] paths2 = split2 = StrUtils.split(classpath, File.pathSeparatorChar);
            for (String path2 : split2) {
                path2 = path2.trim();
                if (path2.length() > 0) {
                    final URL url2 = URLUtils.fromFile(path2);
                    urls.add(url2);
                }
            }
        }
        try {
            final Enumeration<URL> paths3 = classLoader.getResources("META-INF/MANIFEST.MF");
            while (paths3.hasMoreElements()) {
                final URL url3 = paths3.nextElement();
                final File file = URLUtils.toFileObject(url3);
                urls.add(file.toURI().toURL());
            }
        }
        catch (Exception e) {
            throw new RuntimeException(e);
        }
        final Iterator<URL> it = urls.iterator();
        while (it.hasNext()) {
            final String path3 = it.next().getPath();
            if (path3.contains("/jre/lib/")) {
                it.remove();
            }
        }
        return urls;
    }
    
    private static Collection<URL> getClasspathURLs(final ClassLoader classLoader, final String packageName) {
        if (packageName == null) {
            throw new IllegalArgumentException("PackageName must be not null.");
        }
        final Collection<URL> urls = new ArrayList<URL>();
        final String dirname = packageName.replace('.', '/');
        try {
            final Enumeration<URL> dirs = classLoader.getResources(dirname);
            while (dirs.hasMoreElements()) {
                urls.add(dirs.nextElement());
            }
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }
        return urls;
    }
    
    public static Collection<URL> getClasspathURLs(final String packageName) {
        final ClassLoader classLoader = getClassLoader();
        if (StrUtils.isBlank(packageName)) {
            return getClasspathURLs(classLoader);
        }
        return getClasspathURLs(classLoader, packageName);
    }
    
    public static void main(final String[] args) {
    }
}
