package xgs.net.utils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

public abstract class DateUtils
{
    public static final String PATTERN_DATETIME = "yyyy-MM-dd HH:mm:ss";
    public static final String PATTERN_DATE = "yyyy-MM-dd";
    public static final String PATTERN_TIME = "HH:mm:ss";
    
    public static Date setYears(final Date date, final int amount) {
        return set(date, 1, amount);
    }
    
    public static Date setMonths(final Date date, final int amount) {
        return set(date, 2, amount);
    }
    
    public static Date setWeeks(final Date date, final int amount) {
        return set(date, 3, amount);
    }
    
    public static Date setDays(final Date date, final int amount) {
        return set(date, 5, amount);
    }
    
    public static Date setHours(final Date date, final int amount) {
        return set(date, 11, amount);
    }
    
    public static Date setMinutes(final Date date, final int amount) {
        return set(date, 12, amount);
    }
    
    public static Date setSeconds(final Date date, final int amount) {
        return set(date, 13, amount);
    }
    
    public static Date setMilliseconds(final Date date, final int amount) {
        return set(date, 14, amount);
    }
    
    private static Date set(final Date date, final int calendarField, final int amount) {
        if (date == null) {
            throw new IllegalArgumentException("The date must not be null");
        }
        final Calendar c = Calendar.getInstance();
        c.setLenient(false);
        c.setTime(date);
        c.set(calendarField, amount);
        return c.getTime();
    }
    
    public static Date add(final Date date, final int calendarField, final int amount) {
        if (date == null) {
            throw new IllegalArgumentException("The date must not be null");
        }
        final Calendar c = Calendar.getInstance();
        c.setLenient(false);
        c.setTime(date);
        c.add(calendarField, amount);
        return c.getTime();
    }
    
    public static Date getStartDate(final Date date) {
        if (date == null) {
            throw new IllegalArgumentException("The date must not be null");
        }
        final Calendar calendar = Calendar.getInstance();
        calendar.setLenient(false);
        calendar.setTime(date);
        calendar.set(11, 0);
        calendar.set(12, 0);
        calendar.set(13, 0);
        calendar.set(14, 0);
        return calendar.getTime();
    }
    
    public static Date getEndDate(final Date date) {
        if (date == null) {
            throw new IllegalArgumentException("The date must not be null");
        }
        final Calendar calendar = Calendar.getInstance();
        calendar.setLenient(false);
        calendar.setTime(date);
        calendar.set(11, 23);
        calendar.set(12, 59);
        calendar.set(13, 59);
        calendar.set(14, 59);
        return calendar.getTime();
    }
    
    public static String getStartDateStr(final Date date) {
        return format(getStartDate(date), "yyyy-MM-dd HH:mm:ss");
    }
    
    public static String getEndDateStr(final Date date) {
        return format(getEndDate(date), "yyyy-MM-dd HH:mm:ss");
    }
    
    public static Long getLongDateByStr(final String dateStr) {
        return parse(dateStr, "yyyy-MM-dd HH:mm:ss").getTime();
    }
    
    public static String getBeforeStartDateStr() {
        return format(getStartDate(getBeforeDate()), "yyyy-MM-dd HH:mm:ss");
    }
    
    public static Date getBeforeDate() {
        final Date date = new Date();
        final Calendar calendar = Calendar.getInstance();
        calendar.setLenient(false);
        calendar.setTime(date);
        calendar.set(5, calendar.get(5) - 1);
        return calendar.getTime();
    }
    
    public static String getBeforeEndDateStr() {
        return format(getEndDate(getBeforeDate()), "yyyy-MM-dd HH:mm:ss");
    }
    
    public static Long getBeforeEndDate() {
        return getEndDate(getBeforeDate()).getTime();
    }
    
    public static Long getBeforeStartDate() {
        return getStartDate(getBeforeDate()).getTime();
    }
    
    public static String format(final Date date, final String pattern) {
        final SimpleDateFormat format = new SimpleDateFormat(pattern);
        return format.format(date);
    }
    
    public static Date parse(final String dateStr, final String pattern) {
        final SimpleDateFormat format = new SimpleDateFormat(pattern);
        try {
            return format.parse(dateStr);
        }
        catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }
    
    public static Date parse(final Long dateStr, final String pattern) {
        final SimpleDateFormat format = new SimpleDateFormat(pattern);
        try {
            return format.parse(format.format(new Date(dateStr)));
        }
        catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }
    
    public static String format(final Long dateStr, final String pattern) {
        final SimpleDateFormat format = new SimpleDateFormat(pattern);
        try {
            return format.format(new Date(dateStr));
        }
        catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
    
    public static boolean before(final String dateStr, final String dateStr1) {
        final Date date = parse(dateStr, "yyyy-MM-dd HH:mm:ss");
        final Date date2 = parse(dateStr1, "yyyy-MM-dd HH:mm:ss");
        return date.before(date2);
    }
    
    public static boolean before(final String dateStr) {
        final Date date = parse(dateStr, "yyyy-MM-dd HH:mm:ss");
        final Date date2 = new Date();
        return date.before(date2);
    }
    
    public static List<String> getTimesByDate(final String date) {
        final List<String> result = new ArrayList<String>();
        final Calendar calendar = Calendar.getInstance();
        for (int i = 0; i < 24; ++i) {
            calendar.setLenient(false);
            calendar.setTime(parse(date, "yyyy-MM-dd HH:mm:ss"));
            result.add(getTimeStart(calendar, i));
            result.add(getTimeEnd(calendar, i));
        }
        return result;
    }
    
    public static String getTimeStart(final Calendar calendar, final Integer hour) {
        calendar.set(11, hour);
        calendar.set(12, 0);
        calendar.set(13, 0);
        calendar.set(14, 0);
        return format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
    }
    
    public static String getTimeStart(final String dateStr) {
        final Calendar calendar = Calendar.getInstance();
        calendar.setLenient(false);
        calendar.setTime(parse(dateStr, "yyyy-MM-dd HH:mm:ss"));
        return getTimeStart(calendar, calendar.get(11));
    }
    
    public static String getTimeEnd(final String dateStr) {
        final Calendar calendar = Calendar.getInstance();
        calendar.setLenient(false);
        calendar.setTime(parse(dateStr, "yyyy-MM-dd HH:mm:ss"));
        return getTimeEnd(calendar, calendar.get(11));
    }
    
    public static String getTimeEnd(final Calendar calendar, final Integer hour) {
        calendar.set(11, hour);
        calendar.set(12, 59);
        calendar.set(13, 59);
        calendar.set(14, 59);
        return format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
    }
}
