package xgs.net.utils;

import java.io.*;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;

public abstract class FileUtils
{
    public static List<File> list(final String path) {
        final File file = new File(path);
        return list(file, TrueFilter.TRUE);
    }
    
    public static List<File> list(final String path, final FilenameFilter filter) {
        final File file = new File(path);
        return list(file, filter);
    }
    
    public static List<File> list(final File file) {
        final List<File> fileList = new ArrayList<File>();
        return list(file, fileList, TrueFilter.TRUE);
    }
    
    public static List<File> list(final File file, final FilenameFilter filter) {
        final List<File> fileList = new ArrayList<File>();
        return list(file, fileList, filter);
    }
    
    private static List<File> list(final File file, final List<File> fileList, final FilenameFilter filter) {
        if (file.isDirectory()) {
            final File[] listFiles;
            final File[] files = listFiles = file.listFiles();
            for (final File _file : listFiles) {
                list(_file, fileList, filter);
            }
        }
        else {
            final File dir = file.getParentFile();
            final String name = file.getName();
            final boolean accept = filter.accept(dir, name);
            if (file.exists() && accept) {
                fileList.add(file);
            }
        }
        return fileList;
    }
    
    public static String getTempDir() {
        return System.getProperty("java.io.tmpdir");
    }
    
    public static String readToString(final File file) throws IOException {
        return readToString(file, Charsets.UTF_8);
    }
    
    public static String readToString(final File file, final Charset encoding) throws IOException {
        InputStream in = null;
        try {
            in = new FileInputStream(file);
            return IOUtils.toString(in, encoding);
        }
        finally {
            IOUtils.closeQuietly(in);
        }
    }
    
    public static void writeToFile(final File file, final String data) throws IOException {
        writeToFile(file, data, Charsets.UTF_8, false);
    }
    
    public static void writeToFile(final File file, final String data, final boolean append) throws IOException {
        writeToFile(file, data, Charsets.UTF_8, append);
    }
    
    public static void writeToFile(final File file, final String data, final Charset encoding) throws IOException {
        writeToFile(file, data, encoding, false);
    }
    
    public static void writeToFile(final File file, final String data, final Charset encoding, final boolean append) throws IOException {
        OutputStream out = null;
        try {
            out = new FileOutputStream(file, append);
            IOUtils.write(data, out, encoding);
        }
        finally {
            IOUtils.closeQuietly(out);
        }
    }
    
    public static void main(final String[] args) {
        try {
            System.out.println(getTempDir());
            System.out.println(readToString(new File("D://TT.txt")));
            writeToFile(new File("D://TT.txt"), "MSG");
        }
        catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    public static class TrueFilter implements FilenameFilter, Serializable
    {
        private static final long serialVersionUID = -6420452043795072619L;
        public static final TrueFilter TRUE;
        
        @Override
        public boolean accept(final File dir, final String name) {
            return true;
        }
        
        static {
            TRUE = new TrueFilter();
        }
    }
}
