package xgs.net.utils;

import java.security.MessageDigest;
import java.security.SecureRandom;

public class HashUtils
{
    private static final SecureRandom random;
    private static final char[] HEX_DIGITS;
    private static final char[] CHAR_ARRAY;
    
    public static String md5(final String srcStr) {
        return hash("MD5", srcStr);
    }
    
    public static String sha1(final String srcStr) {
        return hash("SHA-1", srcStr);
    }
    
    public static String sha256(final String srcStr) {
        return hash("SHA-256", srcStr);
    }
    
    public static String sha384(final String srcStr) {
        return hash("SHA-384", srcStr);
    }
    
    public static String sha512(final String srcStr) {
        return hash("SHA-512", srcStr);
    }
    
    public static String hash(final String algorithm, final String srcStr) {
        try {
            final MessageDigest md = MessageDigest.getInstance(algorithm);
            final byte[] bytes = md.digest(srcStr.getBytes("utf-8"));
            return toHex(bytes);
        }
        catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
    
    private static String toHex(final byte[] bytes) {
        final StringBuilder ret = new StringBuilder(bytes.length * 2);
        for (int i = 0; i < bytes.length; ++i) {
            ret.append(HashUtils.HEX_DIGITS[bytes[i] >> 4 & 0xF]);
            ret.append(HashUtils.HEX_DIGITS[bytes[i] & 0xF]);
        }
        return ret.toString();
    }
    
    public static String generateSalt(final int saltLength) {
        final StringBuilder salt = new StringBuilder();
        for (int i = 0; i < saltLength; ++i) {
            salt.append(HashUtils.CHAR_ARRAY[HashUtils.random.nextInt(HashUtils.CHAR_ARRAY.length)]);
        }
        return salt.toString();
    }
    
    public static String generateSaltForSha256() {
        return generateSalt(32);
    }
    
    public static String generateSaltForSha512() {
        return generateSalt(64);
    }
    
    public static boolean slowEquals(final byte[] a, final byte[] b) {
        if (a == null || b == null) {
            return false;
        }
        int diff = a.length ^ b.length;
        for (int i = 0; i < a.length && i < b.length; ++i) {
            diff |= (a[i] ^ b[i]);
        }
        return diff == 0;
    }
    
    static {
        random = new SecureRandom();
        HEX_DIGITS = "0123456789abcdef".toCharArray();
        CHAR_ARRAY = "_-0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ".toCharArray();
    }
}
