package xgs.net.utils;

import java.io.*;
import java.nio.charset.Charset;

public abstract class IOUtils
{
    private static final int DEFAULT_BUFFER_SIZE = 4096;
    
    public static void closeQuietly(final Closeable closeable) {
        try {
            if (closeable != null) {
                closeable.close();
            }
        }
        catch (IOException ex) {}
    }
    
    public static String toString(final InputStream input) throws IOException {
        return toString(input, Charsets.UTF_8);
    }
    
    public static String toString(final InputStream input, final Charset charset) throws IOException {
        final InputStreamReader in = new InputStreamReader(input, charset);
        final StringBuilder out = new StringBuilder();
        final char[] c = new char[4096];
        int n;
        while ((n = in.read(c)) != -1) {
            out.append(new String(c, 0, n));
        }
        closeQuietly(in);
        closeQuietly(input);
        return out.toString();
    }
    
    public static byte[] toByteArray(final InputStream is) {
        final ByteArrayOutputStream out = new ByteArrayOutputStream();
        try {
            final byte[] buffer = new byte[4096];
            int n = 0;
            while (-1 != (n = is.read(buffer))) {
                out.write(buffer, 0, n);
            }
            return out.toByteArray();
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }
        finally {
            closeQuietly(out);
        }
    }
    
    public static void write(final String data, final OutputStream output, final Charset encoding) throws IOException {
        if (data != null) {
            output.write(data.getBytes(encoding));
        }
    }
}
