package xgs.net.utils;

import java.lang.reflect.*;

public class ReflectionUtils
{
    public static IllegalArgumentException convertToUncheckedException(final Exception ex) {
        if (ex instanceof IllegalAccessException || ex instanceof IllegalArgumentException || ex instanceof NoSuchMethodException) {
            throw new IllegalArgumentException("\u53cd\u5c04\u5f02\u5e38", ex);
        }
        throw new IllegalArgumentException(ex);
    }
    
    public static Class getSuperClassGenricType(final Class clazz, final int index) {
        final Type genType = clazz.getGenericSuperclass();
        if (!(genType instanceof ParameterizedType)) {
            System.out.println(clazz.getSimpleName() + "'s superclass not ParameterizedType");
            return Object.class;
        }
        final Type[] params = ((ParameterizedType)genType).getActualTypeArguments();
        if (index >= params.length || index < 0) {
            System.out.println("Index: " + index + ", Size of " + clazz.getSimpleName() + "'s Parameterized Type: " + params.length);
            return Object.class;
        }
        if (!(params[index] instanceof Class)) {
            System.out.println(clazz.getSimpleName() + " not set the actual class on superclass generic parameter");
            return Object.class;
        }
        return (Class)params[index];
    }
    
    public static <T> Class<T> getSuperGenericType(final Class clazz) {
        return (Class<T>)getSuperClassGenricType(clazz, 0);
    }
    
    public static Method getDeclaredMethod(final Object object, final String methodName, final Class<?>[] parameterTypes) {
        Class<?> superClass = object.getClass();
        while (superClass != Object.class) {
            try {
                return superClass.getDeclaredMethod(methodName, parameterTypes);
            }
            catch (NoSuchMethodException ex) {
                superClass = superClass.getSuperclass();
                continue;
            }
        }
        return null;
    }
    
    public static void makeAccessible(final Field field) {
        if (!Modifier.isPublic(field.getModifiers())) {
            field.setAccessible(true);
        }
    }
    
    public static Field getDeclaredField(final Object object, final String filedName) {
        Class<?> superClass = object.getClass();
        while (superClass != Object.class) {
            try {
                return superClass.getDeclaredField(filedName);
            }
            catch (NoSuchFieldException ex) {
                superClass = superClass.getSuperclass();
                continue;
            }
        }
        return null;
    }
    
    public static Object invokeMethod(final Object object, final String methodName, final Class<?>[] parameterTypes, final Object[] parameters) throws InvocationTargetException {
        final Method method = getDeclaredMethod(object, methodName, parameterTypes);
        if (method == null) {
            throw new IllegalArgumentException("Could not find method [" + methodName + "] on target [" + object + "]");
        }
        method.setAccessible(true);
        try {
            return method.invoke(object, parameters);
        }
        catch (IllegalAccessException e) {
            System.out.println("\u4e0d\u53ef\u80fd\u629b\u51fa\u7684\u5f02\u5e38:" + e.getMessage());
            return null;
        }
    }
    
    public static Object invokeMethod(final Object object, final Method method, final Object[] parameters) throws InvocationTargetException {
        method.setAccessible(true);
        try {
            if (parameters != null) {
                return method.invoke(object, parameters);
            }
            return method.invoke(object, new Object[0]);
        }
        catch (IllegalAccessException e) {
            System.out.println("\u4e0d\u53ef\u80fd\u629b\u51fa\u7684\u5f02\u5e38:" + e.getMessage());
            return null;
        }
    }
    
    public static Method getMethodLikeName(final Object object, final String method) {
        for (Class<?> superClass = object.getClass(); superClass != Object.class; superClass = superClass.getSuperclass()) {
            try {
                final Method[] declaredMethods;
                final Method[] methods = declaredMethods = superClass.getDeclaredMethods();
                for (final Method me : declaredMethods) {
                    if (me.getName().toLowerCase().equals(method.toLowerCase())) {
                        return me;
                    }
                }
            }
            catch (Exception ex) {}
        }
        return null;
    }
    
    public static void setFieldValue(final Object object, final String fieldName, final Object value) {
        final Field field = getDeclaredField(object, fieldName);
        if (field == null) {
            throw new IllegalArgumentException("Could not find field [" + fieldName + "] on target [" + object + "]");
        }
        makeAccessible(field);
        try {
            field.set(object, value);
        }
        catch (IllegalAccessException e) {
            System.out.println("\u4e0d\u53ef\u80fd\u629b\u51fa\u7684\u5f02\u5e38:" + e.getMessage());
        }
    }
    
    public static Object getFieldValue(final Object object, final String fieldName) {
        final Field field = getDeclaredField(object, fieldName);
        if (field == null) {
            throw new IllegalArgumentException("Could not find field [" + fieldName + "] on target [" + object + "]");
        }
        makeAccessible(field);
        Object result = null;
        try {
            result = field.get(object);
        }
        catch (IllegalAccessException e) {
            System.out.println("\u4e0d\u53ef\u80fd\u629b\u51fa\u7684\u5f02\u5e38:" + e.getMessage());
        }
        return result;
    }
}
