package xgs.net.utils;

import java.text.MessageFormat;
import java.util.*;
import java.util.stream.Collectors;

public class StrUtils
{
    private static final String _INT = "0123456789";
    private static final String _STR = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
    private static final String _ALL = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
    private static final Random RANDOM;
    
    public static String firstCharToLowerCase(final String str) {
        final char firstChar = str.charAt(0);
        if (firstChar >= 'A' && firstChar <= 'Z') {
            final char[] charArray;
            final char[] arr = charArray = str.toCharArray();
            final int n = 0;
            charArray[n] += ' ';
            return new String(arr);
        }
        return str;
    }
    
    public static String firstCharToUpperCase(final String str) {
        final char firstChar = str.charAt(0);
        if (firstChar >= 'a' && firstChar <= 'z') {
            final char[] charArray;
            final char[] arr = charArray = str.toCharArray();
            final int n = 0;
            charArray[n] -= ' ';
            return new String(arr);
        }
        return str;
    }
    
    public static boolean isBlank(final String str) {
        if (str == null) {
            return true;
        }
        final int len = str.length();
        if (len == 0) {
            return true;
        }
        int i = 0;
        while (i < len) {
            switch (str.charAt(i)) {
                case '\t':
                case '\n':
                case '\r':
                case ' ': {
                    ++i;
                    continue;
                }
                default: {
                    return false;
                }
            }
        }
        return true;
    }
    
    public static boolean notBlank(final String str) {
        return !isBlank(str);
    }
    
    public static boolean notBlank(final String... strings) {
        if (strings == null) {
            return false;
        }
        for (final String str : strings) {
            if (isBlank(str)) {
                return false;
            }
        }
        return true;
    }
    
    public static boolean notNull(final Object... paras) {
        if (paras == null) {
            return false;
        }
        for (final Object obj : paras) {
            if (obj == null) {
                return false;
            }
        }
        return true;
    }
    
    public static String toCamelCase(String stringWithUnderline) {
        if (stringWithUnderline.indexOf(95) == -1) {
            return stringWithUnderline;
        }
        stringWithUnderline = stringWithUnderline.toLowerCase();
        final char[] fromArray = stringWithUnderline.toCharArray();
        final char[] toArray = new char[fromArray.length];
        int j = 0;
        for (int i = 0; i < fromArray.length; ++i) {
            if (fromArray[i] == '_') {
                if (++i < fromArray.length) {
                    toArray[j++] = Character.toUpperCase(fromArray[i]);
                }
            }
            else {
                toArray[j++] = fromArray[i];
            }
        }
        return new String(toArray, 0, j);
    }
    
    public static String join(final String... stringArray) {
        final StringBuilder sb = new StringBuilder();
        for (final String s : stringArray) {
            sb.append(s);
        }
        return sb.toString();
    }
    
    public static String join(final String[] stringArray, final String separator) {
        final StringBuilder sb = new StringBuilder();
        for (int i = 0; i < stringArray.length; ++i) {
            if (i > 0) {
                sb.append(separator);
            }
            sb.append(stringArray[i]);
        }
        return sb.toString();
    }
    
    public static boolean slowEquals(final String a, final String b) {
        final byte[] aBytes = (byte[])((a != null) ? a.getBytes() : null);
        final byte[] bBytes = (byte[])((b != null) ? b.getBytes() : null);
        return HashUtils.slowEquals(aBytes, bBytes);
    }
    
    public static boolean equals(final String a, final String b) {
        return (a == null) ? (b == null) : a.equals(b);
    }
    
    public static String getRandomUUID() {
        return UUID.randomUUID().toString().replace("-", "");
    }
    
    public static String trim(final String str) {
        return (str == null) ? null : str.trim();
    }
    
    public static String trimToEmpty(final String str) {
        if (str == null) {
            return "";
        }
        return str.trim();
    }
    
    public static String sqlHolder(final int size) {
        final String[] paras = new String[size];
        Arrays.fill(paras, "?");
        return join(paras, ",");
    }
    
    public static String format(final String s, final Object... args) {
        return MessageFormat.format(s, args);
    }
    
    public static String escapeHtml(String html) {
        if (html == null) {
            return "";
        }
        html = html.replace("<", "&lt;");
        html = html.replace(">", "&gt;");
        html = html.replace("\"", "&quot;");
        html = html.replace("\n", "</br>");
        return html;
    }
    
    public static String cleanChars(final String txt) {
        return txt.replaceAll("[ \u3000`��\u2022\ufffd\u0001\\f\\t\\v\\s]", "");
    }
    
    public static String random(final int count) {
        return random(count, RandomType.ALL);
    }
    
    public static String random(final int count, final RandomType randomType) {
        if (count == 0) {
            return "";
        }
        if (count < 0) {
            throw new IllegalArgumentException("Requested random string length " + count + " is less than 0.");
        }
        final char[] buffer = new char[count];
        for (int i = 0; i < count; ++i) {
            if (randomType.equals(RandomType.INT)) {
                buffer[i] = "0123456789".charAt(StrUtils.RANDOM.nextInt("0123456789".length()));
            }
            else if (randomType.equals(RandomType.STRING)) {
                buffer[i] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz".charAt(StrUtils.RANDOM.nextInt("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz".length()));
            }
            else {
                buffer[i] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz".charAt(StrUtils.RANDOM.nextInt("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz".length()));
            }
        }
        return new String(buffer);
    }
    
    public static String[] split(final String str) {
        return split(str, ',');
    }
    
    public static String[] split(final String str, final char delimiter) {
        final List<String> results = new ArrayList<String>();
        int ipos;
        int lastpos;
        for (ipos = 0, lastpos = 0; (ipos = str.indexOf(delimiter, lastpos)) != -1; lastpos = ipos + 1) {
            results.add(str.substring(lastpos, ipos));
        }
        if (lastpos < str.length()) {
            results.add(str.substring(lastpos));
        }
        return results.toArray(new String[results.size()]);
    }
    
    public static String joinSymbolBySize(final Integer size, final String str, final String mosaicStr) {
        final String[] result = new String[(int)size];
        for (int i = 0; i < result.length; ++i) {
            result[i] = str;
        }
        return join(result, mosaicStr);
    }
    
    public static List<Long> splitToListLong(final String ids) {
        List<Long> lists = null;
        if (isBlank(ids)) {
            lists = new ArrayList<Long>();
        }
        else {
            lists = Arrays.stream(ids.split(",")).map(Long::parseLong).collect(Collectors.toList());
        }
        return lists;
    }
    
    static {
        RANDOM = new Random();
    }
    
    public enum RandomType
    {
        INT, 
        STRING, 
        ALL;
    }
}
