package xgs.net.utils;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLDecoder;
import java.net.URLEncoder;

public class URLUtils
{
    public static final String FILE_PROTOCOL = "file";
    public static final String JAR_PROTOCOL = "jar";
    public static final String ZIP_PROTOCOL = "zip";
    public static final String VFS_PROTOCOL = "vfs";
    public static final String FILE_PROTOCOL_PREFIX = "file:";
    public static final String JAR_PROTOCOL_PREFIX = "jar:";
    public static final String ZIP_PROTOCOL_PREFIX = "zip:";
    public static final String VFS_PROTOCOL_PREFIX = "vfs:";
    public static final String JAR_FILE_SEPARATOR = "!/";
    
    public static String encode(final String url) {
        try {
            return URLEncoder.encode(url, Charsets.UTF_8.name());
        }
        catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
    }
    
    public static String decode(final String url) {
        try {
            return URLDecoder.decode(url, Charsets.UTF_8.name());
        }
        catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
    }
    
    public static URL fromFile(final String file) {
        return fromFile(new File(file));
    }
    
    public static URL fromFile(final File file) {
        try {
            return file.toURI().toURL();
        }
        catch (MalformedURLException e) {
            throw new RuntimeException(e);
        }
    }
    
    public static File toFileObject(final URL url) {
        if (url == null) {
            return null;
        }
        return new File(toFilePath(url));
    }
    
    public static String toFilePath(final URL url) {
        if (url == null) {
            return null;
        }
        final String protocol = url.getProtocol();
        String file = url.getPath();
        try {
            file = URLDecoder.decode(file, "utf-8");
        }
        catch (UnsupportedEncodingException ex) {}
        if ("file".equals(protocol)) {
            return file;
        }
        if ("jar".equals(protocol) || "zip".equals(protocol)) {
            final int ipos = file.indexOf("!/");
            if (ipos > 0) {
                file = file.substring(0, ipos);
            }
            if (file.startsWith("file:")) {
                file = file.substring("file:".length());
            }
            return file;
        }
        if ("vfs".equals(protocol)) {
            final int ipos = file.indexOf("!/");
            if (ipos > 0) {
                file = file.substring(0, ipos);
            }
            else if (file.endsWith("/")) {
                file = file.substring(0, file.length() - 1);
            }
            return file;
        }
        return file;
    }
    
    public static void main(final String[] args) {
        final URL ttUrl = fromFile("D://TT.txt");
        System.out.println(toFilePath(ttUrl));
    }
}
