package xgs.net.utils;

public class Version
{
    private static final String delimiter = "\\.";
    private String version;
    private boolean complete;
    
    private Version() {
        this.complete = true;
    }
    
    private Version(final String version) {
        this.complete = true;
        this.version = version;
    }
    
    public Version incomplete() {
        this.complete = false;
        return this;
    }
    
    public static Version of(final String version) {
        return new Version(version);
    }
    
    public boolean eq(final String version) {
        return this.compare(version) == 0;
    }
    
    public boolean gt(final String version) {
        return this.compare(version) > 0;
    }
    
    public boolean gte(final String version) {
        return this.compare(version) >= 0;
    }
    
    public boolean lt(final String version) {
        return this.compare(version) < 0;
    }
    
    public boolean lte(final String version) {
        return this.compare(version) <= 0;
    }
    
    private int compare(final String version) {
        return compare(this.version, version, this.complete);
    }
    
    private static int compare(final String v1, final String v2, final boolean complete) {
        if (v1.equals(v2)) {
            return 0;
        }
        final String[] v1s = v1.split("\\.");
        final String[] v2s = v2.split("\\.");
        for (int len = complete ? Math.max(v1s.length, v2s.length) : Math.min(v1s.length, v2s.length), i = 0; i < len; ++i) {
            final String c1 = (null == v1s[i]) ? "" : v1s[i];
            final String c2 = (null == v2s[i]) ? "" : v2s[i];
            final int result = c1.compareTo(c2);
            if (result != 0) {
                return result;
            }
        }
        return 0;
    }
}
