package xgs.net.utils;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public final class WebUtils
{
    private static String USER_COOKIE_KEY;
    private static String USER_COOKIE_SECRET;
    
    public static String pwdEncode(final String pwd) {
        return HashUtils.md5(HashUtils.sha1(pwd));
    }
    
    public static void setUserKey(final String userKey) {
        if (StrUtils.isBlank(userKey)) {
            return;
        }
        WebUtils.USER_COOKIE_KEY = userKey;
    }
    
    public static void setUserSecret(final String userSecret) {
        if (StrUtils.isBlank(userSecret)) {
            return;
        }
        WebUtils.USER_COOKIE_SECRET = userSecret;
    }
    
    public static String getCookie(final HttpServletRequest request, final String key) {
        final Cookie[] cookies = request.getCookies();
        if (null != cookies) {
            for (final Cookie cookie : cookies) {
                if (key.equals(cookie.getName())) {
                    return cookie.getValue();
                }
            }
        }
        return null;
    }
    
    public static void removeCookie(final HttpServletResponse response, final String key) {
        setCookie(response, key, null, null, 0);
    }
    
    public static void setCookie(final HttpServletResponse response, final String name, final String value, final int maxAgeInSeconds) {
        setCookie(response, name, value, null, maxAgeInSeconds);
    }
    
    public static void setCookie(final HttpServletResponse response, final String name, final String value, final String domain, final int maxAgeInSeconds) {
        final Cookie cookie = new Cookie(name, value);
        cookie.setPath("/");
        cookie.setMaxAge(maxAgeInSeconds);
        if (StrUtils.notBlank(domain)) {
            cookie.setDomain(domain);
        }
        try {
            cookie.setHttpOnly(true);
        }
        catch (Exception ex) {}
        response.addCookie(cookie);
    }
    
    public static String getUserAgent(final HttpServletRequest request) {
        return request.getHeader("User-Agent");
    }
    
    public static String getIP(final HttpServletRequest request) {
        String ip = request.getHeader("X-Requested-For");
        if (StrUtils.isBlank(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("X-Forwarded-For");
        }
        if (StrUtils.isBlank(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("Proxy-Client-IP");
        }
        if (StrUtils.isBlank(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (StrUtils.isBlank(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("HTTP_CLIENT_IP");
        }
        if (StrUtils.isBlank(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("HTTP_X_FORWARDED_FOR");
        }
        if (StrUtils.isBlank(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
        }
        return StrUtils.isBlank(ip) ? null : ip.split(",")[0];
    }
    
    static {
        WebUtils.USER_COOKIE_KEY = "uid";
        WebUtils.USER_COOKIE_SECRET = "&#%!&*";
    }
}
