package xgs.net.utils.tree;

import xgs.net.annotation.TreeNodeId;
import xgs.net.annotation.TreeNodeName;
import xgs.net.annotation.TreeNodePid;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

public class TreeHelper {

    /**
     * author SHS
     * 将用户的数据转化为树形数据
     *
     * @param datas 用户数据
     * 注解 + 反射 使之适应多种需求
     */
    public static <T> List<TreeNode> convertDatas2Nodes(List<T> datas) throws Exception
    {
        List<TreeNode> nodes = new ArrayList<TreeNode>();
        TreeNode node = null;
        for (T t : datas)
        {
            String id = "";
            String pid = "";
            String label = null;


            node = new TreeNode() {};
            Class clazz = t.getClass();
            //获取对应注解数据
            Field[] fields = clazz.getDeclaredFields();
            for (Field field : fields)
            {
                if (field.getAnnotation(TreeNodeId.class) != null)
                {
                    field.setAccessible(true);
                    id = field.get(t).toString();
                }
                if (field.getAnnotation(TreeNodePid.class) != null)
                {
                    field.setAccessible(true);
                    pid = field.get(t).toString();
                }
                if (field.getAnnotation(TreeNodeName.class) != null)
                {
                    field.setAccessible(true);
                    label = (String) field.get(t);
                }
            }
            node = new TreeNode(id,pid,label){};
            nodes.add(node);
        }// for end

        /**
         * 设置Node间的节点关系
         */
        for (int i = 0; i < nodes.size(); i++)
        {
            TreeNode n = nodes.get(i);

            //判断首个父节点
            int s = 0;
            for (int j = i ; j < nodes.size(); j++)
            {
                TreeNode m = nodes.get(j);
                if (m.getpId().equals(n.getpId()))
                {
                    n.getChildren().add(m);
                    if(s != 0)
                        m.setParent(n);
                }
                s++;
            }
            s = 0;
        }

        return nodes;
    }

    /*
    * 排序完成后的树
    * */
    public static <T> List<TreeNode> getTreeNode(List<T> datas) throws Exception {
        List<TreeNode> nodes = convertDatas2Nodes(datas);
        List<TreeNode> result = new ArrayList<>();
        //父节点ID
        int index = 1;
        for (TreeNode node : nodes){
            if(node.getParent() == null){
                TreeNode treeNode = new TreeNode() {};
                List<TreeNode> childrenNode = new ArrayList<>();
                treeNode.setId(String.valueOf(index));
                treeNode.setLabel(node.getpId());
                treeNode.setpId(node.getpId());
                treeNode.setParent(null);
                treeNode.setChildren(getChildrenNode(node.getChildren()));
                index ++;
                result.add(treeNode);
            }
        }
        return result;
    }


    /*
    * 获取子节点
    * */
    public static List<TreeNode> getChildrenNode(List<TreeNode> children){
        List<TreeNode> childrenNode = children;
        List<TreeNode> result = new ArrayList<>();
        for (TreeNode node : childrenNode){
            TreeNode treeNode = new TreeNode() {};
            treeNode.setId(node.getId());
            treeNode.setpId(node.getpId());
            treeNode.setLabel(node.getLabel());
            result.add(treeNode);
        }
        return result;
    }
}
