package xgs.net.server;

import com.jfinal.kit.Prop;
import com.jfinal.kit.PropKit;
import com.jfinal.kit.StrKit;
import com.jfinal.weixin.sdk.api.PaymentApi;
import com.jfinal.weixin.sdk.api.SnsAccessToken;
import com.jfinal.weixin.sdk.api.SnsAccessTokenApi;
import com.jfinal.weixin.sdk.kit.PaymentKit;
import xgs.net.annotation.Service;
import xgs.net.entity.Msg;
import xgs.net.utils.URLUtils;
import xgs.net.utils.XmlHelper;

import java.util.HashMap;
import java.util.Map;


@Service
public class PayService {

    private Prop prop  = PropKit.use("pay.properties");


    /*
    * create shs
    * createdate 20181231
    * 统一下单
    * @parma tradeId 商户订单Id
    * @parma code 微信登录Id
    * @parma totalFee 支付金额
    * */
    public Msg pushOrder(String tradeId,String openId,Integer totalFee){
        Msg msg = new Msg();

        Map<String, String> params = new HashMap<String, String>();
        params.put("appid", this.prop.get("wx.appId"));
        params.put("mch_id", this.prop.get("wx.partner"));
        params.put("body", "rew");
        params.put("out_trade_no", tradeId);
        params.put("total_fee", totalFee+"");
        params.put("attach", "公众号支付");

        String ip = this.prop.get("wx.spbillCreateIp");//IpKit.getRealIp(this.getRequest());
        if (StrKit.isBlank(ip)) {
            ip = "127.0.0.1";
        }


        params.put("spbill_create_ip", ip);
        params.put("nonce_str", StrKit.getRandomUUID());
        params.put("notify_url", this.prop.get("wx.notifyUrl"));
        if (openId != null) {
            params.put("openid", openId);
            params.put("trade_type", PaymentApi.TradeType.JSAPI.name());
        }else{
            params.put("trade_type", PaymentApi.TradeType.APP.name());
        }

        String sign = PaymentKit.createSign(params,this.prop.get("wx.parentKey"));
        params.put("sign", sign);


        String orderInfo = PaymentApi.pushOrder(params);
        XmlHelper order = XmlHelper.of(orderInfo);

        if(!order.getString("//return_msg").equals("OK")){
            msg.setCode(54421);
            msg.setMsg("订单创建失败");
            msg.setData(null);
            return msg;
        }

        long timeStampSec = System.currentTimeMillis()/1000;
        String timestamp = String.format("%010d", timeStampSec);
        //返回数据
        Map<String,String> res = new HashMap();
        res.put("appId",this.prop.get("wx.appId"));
        res.put("nonceStr",order.getString("//nonce_str"));
        res.put("package","prepay_id="+order.getString("//prepay_id"));
        res.put("signType","MD5");
        res.put("timeStamp",timestamp);
        String paySign = PaymentKit.createSign(res,this.prop.get("wx.parentKey"));
        res.put("paySign",paySign);

        msg.setData(res);
        return msg;
    }

    /*
     * create shs
     * createdate 20181231
     * 关闭订单
     * */
    public Msg closeOrder(String transactionId){

        Msg msg = new Msg();

        Map<String,String> res = PaymentApi.closeOrder(this.prop.get("wx.appId"),this.prop.get("wx.partner"),this.prop.get("wx.parentKey"),transactionId);

        if(res.get("return_code").equals("SUCCESS")){
            msg.setCode(54426);
            msg.setMsg(res.get("return_msg"));
            msg.setData(null);
            return msg;
        }

        msg.setCode(200);
        return msg;
    }


    /*
     * create shs
     * createdate 20181231
     * 退单
     * */
    public Map<String, String> refoundOrder(String tradeId, String refoundId, Integer totalFee, Integer refoundTotalFee){
        Msg msg = new Msg();
        Map<String,String> params = new HashMap<>();
        params.put("appid",this.prop.get("wx.appId"));
        params.put("mch_id",this.prop.get("wx.partner"));
        params.put("nonce_str",StrKit.getRandomUUID());
        params.put("out_trade_no",tradeId);
        params.put("out_refund_no",refoundId);
        params.put("total_fee",totalFee+"");
        params.put("refund_fee",refoundTotalFee+"");
        //params.put("notify_url",this.prop.get("wx.refoundNotifyUrl"));

        String sign = PaymentKit.createSign(params,this.prop.get("wx.parentKey"));
        params.put("sign",sign);
        Map<String,String> res = PaymentApi.refund(params,this.prop.get("wx.parentKey"),this.prop.get("wx.certificate"));
        return res;
    }


    /*
     * create shs
     * createdate 20181231
     * 统一回调地址
     * */
    public String getCodeURl(String URL){
        return SnsAccessTokenApi.getAuthorizeURL(prop.get("wx.appId"), URLUtils.encode(URL),false);
    }


    public String getOpenId(String code){
        SnsAccessToken openId = SnsAccessTokenApi.getSnsAccessToken(this.prop.get("wx.appId"),this.prop.get("wx.appSecret"),code);
        return openId.getOpenid();
    }
}
