import axios from 'axios'
import store from '@/store'
import { getToken, getLocation } from '@/common/js/cookie'
import { Toast } from 'mint-ui'
import { Dialog } from 'vant'
import { router } from '@/router'
// import port from '@/adapter/http/port'

const ajaxUrl =
// production 生产环境
  process.env.NODE_ENV === 'production'
    ? 'http://192.168.5.224:9980'
    : 'http://127.0.0.1:9980'

// create an axios instance
const service = axios.create({
  baseURL: ajaxUrl, // api的base_urlgit
  timeout: 5000 // request timeout
})

// request interceptor
service.interceptors.request.use(
  config => {
    if (getToken()) {
      config.headers['x-token'] = getToken()
    }
    if (getLocation()) {
      let _ = getLocation()
      let data = {
        location: _.location,
        cityCode: _.cityCode,
        adcode: _.adcode,
        text: encodeURI(_.city)
      }
      config.headers['x-location'] = JSON.stringify(data)
    }
    return config
  },
  error => {
    // Toast()
    // Do something with request error
    console.log(error) // for debug
    Promise.reject(error)
  }
)

// respone interceptor
service.interceptors.response.use(
  // response => response,
  /**
   * 下面的注释为通过在response里，自定义code来标示请求状态
   * 当code返回如下情况则说明权限有问题，登出并返回到登录页
   * 如想通过xmlhttprequest来状态码标识 逻辑可写在下面error中
   * 以下代码均为样例，请结合自生需求加以修改，若不需要，则可删除
   */
  response => {
    const res = response.data
    if (res.code !== 200) {
      if (res.code === 999) {
        Dialog.confirm({
          title: '提示',
          message: res.msg + ',登陆?',
          confirmButtonText: '登陆'
        })
          .then(() => {
            router.replace({
              path: `/login`
            })
          })
          .catch(() => {
            router.back()
          })
        store.commit('logout')
      } else {
        Toast({
          message: res.msg,
          position: 'bottom'
        })
      }
      return Promise.reject(res)
    } else {
      return res.data
    }
  },
  error => {
    console.log('err' + error) // for debug
    // Message({
    //   message: error.message,
    //   type: 'error',
    //   duration: 5 * 1000
    // })
    return Promise.reject(error)
  }

)

export default service
