import ImageInterface from './imageInterface'
import xgs, { ximg } from '@/common/js/util'
import axios from 'axios'
import qs from 'qs'
import { Toast, Indicator } from 'mint-ui'

class ImageClass {
  constructor (max, cb) {
    this.max = max
    let input = document.createElement('input')
    input.type = 'file'
    input.accept = 'image/*'
    input.multiple = true
    input.style.display = 'none'
    document.body.appendChild(input)
    if (max <= 1) {
      input.multiple = false
    } else {
      input.multiple = true
    }
    input.onchange = event => {
      this.onChange(event, cb)
    }
    input.onblur = event => {
      console.log(event, 'event')
    }
    let evt = new MouseEvent('click', {
      bubbles: true,
      cancelable: true,
      view: window
    })
    input.dispatchEvent(evt)
    this.input = input
  }
  onChange (event, cb) {
    console.log(event)
    let { files } = event.target
    if (!files.length) {
      return
    }
    files = files.length === 1 ? files[0] : [].slice.call(files, 0)
    if (!files) {
      return
    }
    if (Array.isArray(files)) {
      Promise.all(files.map(this.readFile)).then(contents => {
        let oversize = false
        const payload = files.map((file, index) => {
          if (file.size > this.maxSize) {
            oversize = true
          }
          return {
            file: files[index],
            content: contents[index]
          }
        })
        /* eslint-disable */
        cb(this, payload, oversize)
      })
    } else {
      this.readFile(files).then(content => {
        /* eslint-disable */
        cb(this, [{ file: files, content }], files.size > this.maxSize)
      })
    }
  }
  readFile(file) {
    return new Promise(resolve => {
      const reader = new FileReader()
      reader.onload = event => {
        resolve(event.target.result)
      }
      reader.readAsDataURL(file)
    })
  }
  del() {
    document.body.removeChild(this.input)
    delete this.max
    delete this.input
  }
}

let maxLeng = 1
let config = {
  spinnerType: 'fading-circle',
  text: '正在处理中...1'
}
export class Dev extends ImageInterface {
  openImage(max) {
    return new Promise((resolve, reject) => {
      /* eslint-disable */
      new ImageClass(max, (obj, res) => {
        if (max < res.length) {
          Toast({
            message: '已超过最大选择数',
            position: 'bottom'
          })
          resolve([])
        }
        obj.del()
        maxLeng = res.length
        this.upload(res, [], arr => {
          resolve(arr)
          Indicator.close()
        })
      })
    })
  }
  upload(res, ids, cb) { 
    if (xgs.isArray(res) && res.length !== 0) {
      config = {
        spinnerType: 'fading-circle',
        text: '正在处理中...' + (maxLeng - res.length + 1) + '/' + maxLeng
      }
      Indicator.close()
      Indicator.open(config)
      let item = res[0]
      res.splice(0, 1)
      let _data = {
        base64: item.content
        .replace(/^data:image\/.*?base64,/, '')
        .replace(/[\s*\t\n\r]/g, '')
      }
      let data = qs.stringify(_data)
      axios
        .post(ximg.base64, data, {
          headers: {
            'Content-Type': 'application/x-www-form-urlencoded'
          }
        })
        .then(r => {
          if(r.data.code === 200){
            r.data.data.forEach(item=>{
              if(item.code === 200){
                ids.push(item.md5)
                this.upload(res, ids, cb)
              }
            })
          }
        })
        .catch(r => {
          alert(r)
        })
    } else {
      cb(ids)
    }
  }
  uploadImage(compress, url, paths) {
    return new Promise((resolve, reject) => {})
  }

  preview(current,urls) {
    //return new Promise((resolve, reject) => {})
  }

  avator(size) {
    return new Promise((resolve, reject) => {})
  }
}

let imageUtils = new Dev()
export default imageUtils
