import ImageInterface from './imageInterface'
import wx from 'weixin-js-sdk'
import xgs, { ximg } from '@/common/js/util'
import axios from 'axios'
import qs from 'qs'
import { Indicator } from 'mint-ui'

let maxLeng = 1
let config = {
  spinnerType: 'fading-circle',
  text: '正在处理中...1'
}

export class Weixin extends ImageInterface {
  openImage (max) {
    return new Promise((resolve, reject) => {
      wx.chooseImage({
        count: max, // 默认9
        sizeType: ['oiginal', 'compressed'], // 可以指定是原图还是压缩图，默认二者都有
        sourceType: ['album', 'camera'], // 可以指定来源是相册还是相机，默认二者都有
        success: res => {
          // res.localIds  返回选定照片的本地ID列表，localId可以作为img标签的src属性显示图片
          let localIds = res.localIds
          maxLeng = localIds.length
          this.upload(localIds, [], arr => {
            resolve(arr)
            Indicator.close()
          })
        }
      })
    })
  }
  upload (localIds, ids, cb) {
    if (xgs.isArray(localIds) && localIds.length !== 0) {
      config = {
        spinnerType: 'fading-circle',
        text: '正在处理中...' + (maxLeng - localIds.length + 1) + '/' + maxLeng
      }
      Indicator.close()
      Indicator.open(config)
      wx.getLocalImgData({
        localId: localIds[0],
        success: res => {
          let localData = res.localData
          if (window.__wxjs_is_wkwebview) {
            localData = localData.replace('jgp', 'jpeg')
          } else {
            localData = localData = 'data:image/jpeg;base64,' + localData
          }

          localIds.splice(0, 1)
          let _data = {
            base64: localData.replace(/^data:image\/.*?base64,/, '')
              .replace(/[\s*\t\n\r]/g, '')
          }
          let data = qs.stringify(_data)
          axios
            .post(ximg.base64, data, {
              headers: {
                'Content-Type': 'application/x-www-form-urlencoded'
              }
            })
            .then(r => {
              if (r.data.code === 200) {
                r.data.data.forEach(it => {
                  if (it.code === 200) {
                    ids.push(it.md5)
                  }
                })
              }
              this.upload(localIds, ids, cb)
            })
            .catch(r => {
              alert(r)
            })
        }
      })
    } else {
      cb(ids)
    }
  }

  uploadImage (compress, url, data) {
    return new Promise((resolve, reject) => {
      data.forEach((item, index) => {
        wx.uploadImage({
          localId: item.src, // 需要上传的图片的本地ID，由chooseImage接口获得
          isShowProgressTips: 1, // 默认为1，显示进度提示
          success: function (res) {
            // 在服务器上异步下载
            // var serverId = res.serverId // 返回图片的服务器端ID
          }
        })
      })
    })
  }

  chooseHeader () {
    return new Promise((resolve, reject) => {
      wx.chooseImage({
        count: 1, // 默认9
        sizeType: ['oiginal', 'compressed'], // 可以指定是原图还是压缩图，默认二者都有
        sourceType: ['album', 'camera'], // 可以指定来源是相册还是相机，默认二者都有
        success: res => {
          let localIds = res.localIds
          if (xgs.isArray(localIds) && localIds.length !== 0) {
            wx.getLocalImgData({
              localId: localIds[0],
              success: res => {
                let localData = res.localData
                if (window.__wxjs_is_wkwebview) {
                  localData = localData.replace('jgp', 'jpeg')
                } else {
                  localData = localData = 'data:image/jpeg;base64,' + localData
                }

                localIds.splice(0, 1)
                let _data = {
                  base64: localData.replace(/^data:image\/.*?base64,/, '')
                    .replace(/[\s*\t\n\r]/g, '')
                }
                // let data = qs.stringify(_data)
                resolve(_data)
              }
            })
          } else {
            resolve(null)
          }
        }
      })
    })
  }

  preview (current, urls) {
    wx.previewImage({
      current, // 当前显示图片的http链接
      urls // 需要预览的图片http链接列表
    })
  }

  avator (size) {
    return new Promise((resolve, reject) => {})
  }
}

let imageUtils = new Weixin()
export default imageUtils
