import cookies from 'js-cookie'
import LocationApi from 'LocationApi'
import { Dialog } from 'vant'
import { getBaiduLocation } from '@/common/components/baidumap/index'
import { Toast } from 'mint-ui'
const TokenKey = 'x-token'
const userKey = 'x-user'
const entityKey = 'x-entity'
const cataKey = 'catas'
const LocationKey = 'location'
const foundTypeKey = 'foundType'

export function setUser (user) {
  return cookies.set(userKey, user)
}
export function getUser () {
  return cookies.getJSON(userKey)
}
export function removeUser () {
  return cookies.remove(userKey)
}

export function setEntity (entity) {
  return cookies.set(entityKey, entity)
}
export function getEntity () {
  return cookies.getJSON(entityKey)
}
export function removeEntity () {
  return cookies.remove(entityKey)
}

export function getToken () {
  return cookies.get(TokenKey)
}

export function setToken (token) {
  return cookies.set(TokenKey, token)
}

export function removeToken () {
  return cookies.remove(TokenKey)
}

export function getCataL1 () {
  let temp = []
  let catas = getCatas() || []
  catas.forEach(item => {
    if (item.level === 0) {
      temp.push(Object.assign({}, item))
    }
  })
  return temp
}
export function getCataL2 (id) {
  let temp = []
  let catas = getCatas() || []
  catas.forEach(item => {
    if (item.parent_id === id) {
      temp.push(Object.assign({}, item))
    }
  })
  return temp
}
export function getCatas () {
  let temp = cookies.get(cataKey)
  if (typeof temp === 'string') {
    return JSON.parse(temp)
  }
  return null
}

export function setCatas (data) {
  return cookies.set(cataKey, data)
}

export function removeCatas () {
  return cookies.remove(cataKey)
}

export function getLocation () {
  let loc = cookies.get(LocationKey)
  if (loc) {
    return JSON.parse(loc)
  }
  return null
}

export function changeLocation (city) {
  cookies.set(LocationKey, city)
}

export function setLocation (fullPath) {
  return new Promise((resolve, reject) => {
    LocationApi.getLocation()
      .then(res => {
        if (res.code === 200) {
          let city
          let cityCode
          getBaiduLocation({ lng: res.data.lng, lat: res.data.lat }).then(
            r => {
              if (r.formatted_address && r.formatted_address.indexOf('安徽省六安市寿县') > -1) {
                city = '淮南市'
                cityCode = 250
              } else {
                city = r.city
                cityCode = r.cityCode
              }
              r.city = city
              r.cityCode = cityCode
              r.current = {
                city: city,
                cityCode: cityCode
              }
              let loc = cookies.getJSON(LocationKey)
              if (loc && loc.cityCode) { // 如果为存在以前的location
                if (!fullPath || (fullPath && fullPath !== '/')) { // 但不是首页，则不提醒
                  r.city = loc.city
                  r.cityCode = loc.cityCode
                  cookies.set(LocationKey, r)
                  resolve(r)
                }
                if (loc.cityCode !== r.cityCode && fullPath && fullPath === '/') { // 如果两个城市不是一个城市，且不是首页，提醒
                  Dialog.confirm({
                    title: '提示',
                    message:
                      '您当前所在城市为' + city + '，是否切换到该城市？',
                    confirmButtonText: '切换'
                  })
                    .then(() => {
                      cookies.set(LocationKey, r)
                      resolve(r)
                    })
                    .catch(() => {
                      r.city = loc.city
                      r.cityCode = loc.cityCode
                      cookies.set(LocationKey, r)
                      resolve(r)
                    })
                } else {
                  cookies.set(LocationKey, r)
                  resolve(r)
                }
              } else { // 如果不存在以前的location
                cookies.set(LocationKey, r)
                resolve(r)
              }
            }
          )
        } else {
          let r = {
            city: '兰州',
            cityCode: 36
          }
          cookies.set(LocationKey, r)
          Toast({
            position: 'bottom',
            message: '定位失败，请手动选择'
          })
          reject(r)
        }
      })
      .catch(err => {
        reject(err)
        console.log(err)
      })
  })
}

export function removeLocation () {
  return cookies.remove(LocationKey)
}
export function getFoundType () {
  return cookies.getJSON(foundTypeKey)
}
export function setFoundType (type) {
  return cookies.set(foundTypeKey, type)
}
